@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2020 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2021 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -9,6 +9,8 @@ debugger.
 """
 
 import socket
+import threading
+import contextlib
 
 from DebugUtilities import prepareJsonCommand
 
@@ -30,7 +32,7 @@ def AsyncPendingWrite(file):
     return pending
 
 
-class AsyncFile(object):
+class AsyncFile:
     """
     Class wrapping a socket object with a file interface.
     """
@@ -58,6 +60,7 @@ class AsyncFile(object):
         self.newlines = None
         self.line_buffering = False
         
+        self.writeLock = threading.RLock()
         self.wpending = []
 
     def __checkMode(self, mode):
@@ -99,6 +102,7 @@ class AsyncFile(object):
         """
         Public method to write all pending entries.
         """
+        self.writeLock.acquire()
         while self.wpending:
             try:
                 buf = self.wpending.pop(0)
@@ -106,16 +110,16 @@ class AsyncFile(object):
                 break
             
             try:
-                try:
+                with contextlib.suppress(UnicodeEncodeError,
+                                         UnicodeDecodeError):
                     buf = buf.encode('utf-8', 'backslashreplace')
-                except (UnicodeEncodeError, UnicodeDecodeError):
-                    pass
                 self.sock.sendall(buf)
                 self.nWriteErrors = 0
             except OSError:
                 self.nWriteErrors += 1
                 if self.nWriteErrors > self.maxtries:
                     self.wpending = []    # delete all output
+        self.writeLock.release()
 
     def isatty(self):
         """
@@ -191,8 +195,10 @@ class AsyncFile(object):
         length = int(length)
         data = b''
         while len(data) < length:
-            newData = self.sock.recv(length - len(data))
-            data += newData
+            newByte = self.sock.recv(1)
+            data += newByte
+            if newByte == b'\n':
+                break
         
         # step 2: convert the data
         return data.decode('utf8', 'backslashreplace')
@@ -221,11 +227,7 @@ class AsyncFile(object):
         line = self.sock.recv(size, socket.MSG_PEEK)
 
         eol = line.find(b'\n')
-
-        if eol >= 0:
-            size = eol + 1
-        else:
-            size = len(line)
+        size = eol + 1 if eol >= 0 else len(line)
 
         # Now we know how big the line is, read it for real.
         return self.sock.recv(size).decode('utf8', 'backslashreplace')
@@ -251,7 +253,7 @@ class AsyncFile(object):
             lines.append(line)
 
             if sizehint >= 0:
-                room = room - linelen
+                room -= linelen
 
                 if room <= 0:
                     break
@@ -337,15 +339,18 @@ class AsyncFile(object):
         """
         self.__checkMode('w')
         
+        self.writeLock.acquire()
         if isinstance(s, (bytes, bytearray)):
             # convert to string to send it
             s = repr(s)
         
         cmd = prepareJsonCommand("ClientOutput", {
             "text": s,
+            "debuggerId": "",
         })
         self.wpending.append(cmd)
         self.flush()
+        self.writeLock.release()
     
     def write_p(self, s):
         """
