@@ -9,6 +9,7 @@ Module implementing the debug server.
 
 import os
 import shlex
+import contextlib
 
 from PyQt5.QtCore import pyqtSignal, pyqtSlot, QModelIndex
 from PyQt5.QtNetwork import (
@@ -186,7 +187,7 @@ class DebugServer(QTcpServer):
         @param parent reference to the parent object
         @type QObject
         """
-        super(DebugServer, self).__init__(parent)
+        super().__init__(parent)
         
         self.__originalPathString = originalPathString
         
@@ -425,10 +426,8 @@ class DebugServer(QTcpServer):
         @rtype list of str
         """
         languages = list(self.__debuggerInterfaceRegistry.keys())
-        try:
+        with contextlib.suppress(ValueError):
             languages.remove("None")
-        except ValueError:
-            pass    # it is not in the list
         
         if shellOnly:
             languages = [lang for lang in languages
@@ -508,11 +507,14 @@ class DebugServer(QTcpServer):
         """
         self.running = False
         
-        if not self.passive or not self.passiveClientExited:
-            if self.debuggerInterface and self.debuggerInterface.isConnected():
-                self.shutdownServer()
-                self.debugging = False
-                self.clientGone.emit(unplanned and self.debugging)
+        if (
+            (not self.passive or not self.passiveClientExited) and
+            self.debuggerInterface and
+            self.debuggerInterface.isConnected()
+        ):
+            self.shutdownServer()
+            self.debugging = False
+            self.clientGone.emit(unplanned and self.debugging)
         
         if clType:
             if clType not in self.getSupportedLanguages():
@@ -1485,11 +1487,8 @@ class DebugServer(QTcpServer):
         @param debuggerId ID of the debugger backend
         @type str
         """
-        try:
+        with contextlib.suppress(AttributeError):
             self.debuggerInterface.remoteClientDisassembly(debuggerId)
-        except AttributeError:
-            # remote client doesn't support that
-            pass
     
     def remoteClientSetFilter(self, debuggerId, scope, filterStr):
         """
@@ -1994,12 +1993,9 @@ class DebugServer(QTcpServer):
         @param venvName name of the virtual environment
         @type str
         """
-        try:
+        with contextlib.suppress(KeyError):
             self.__debuggerInterfaceRegistry[clientType][0] = capabilities
             self.clientCapabilities.emit(capabilities, clientType, venvName)
-        except KeyError:
-            # ignore silently
-            pass
     
     def signalClientCompletionList(self, completionList, text, debuggerId):
         """
