@@ -1,16 +1,15 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2015 - 2020 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2015 - 2021 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
 Module implementing a path picker widget.
 """
 
+import enum
 import os
 
-from enum import Enum
-
 from PyQt5.QtCore import pyqtSignal, Qt, QFileInfo, QCoreApplication, QDir
 from PyQt5.QtWidgets import QWidget, QHBoxLayout, QToolButton, QSizePolicy
 
@@ -22,10 +21,12 @@ from .E5ComboBox import E5ClearableComboBox
 import UI.PixmapCache
 
 
-class E5PathPickerModes(Enum):
+class E5PathPickerModes(enum.Enum):
     """
     Class implementing the path picker modes.
     """
+    # TODO: convert these to all uppercase without "Mode" when doing
+    #       the port to PyQt6 (i.e. eric7)
     OpenFileMode = 0
     OpenFilesMode = 1
     SaveFileMode = 2
@@ -69,7 +70,7 @@ class E5PathPickerBase(QWidget):
         @param useLineEdit flag indicating the use of a line edit
         @type bool
         """
-        super(E5PathPickerBase, self).__init__(parent)
+        super().__init__(parent)
         
         self.__lineEditKind = useLineEdit
         
@@ -96,7 +97,7 @@ class E5PathPickerBase(QWidget):
                     "E5PathPickerBase", "Enter Path Name"))
         
         self.__button = QToolButton(self)
-        self.__button.setToolButtonStyle(Qt.ToolButtonIconOnly)
+        self.__button.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonIconOnly)
         self.__button.setIcon(UI.PixmapCache.getIcon("open"))
         
         self.__layout.addWidget(self._editor)
@@ -110,8 +111,9 @@ class E5PathPickerBase(QWidget):
             self._editor.editTextChanged.connect(self.editTextChanged)
         
         self.setFocusProxy(self._editor)
-        self.setFocusPolicy(Qt.StrongFocus)
-        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
+        self.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
+        self.setSizePolicy(QSizePolicy.Policy.Expanding,
+                           QSizePolicy.Policy.Preferred)
         
         self.__button.setEnabled(self.__mode != E5PathPickerModes.NoMode)
     
@@ -510,10 +512,11 @@ class E5PathPickerBase(QWidget):
         directory = self._editorText()
         if not directory and self.__defaultDirectory:
             directory = self.__defaultDirectory
-        if self.__mode == E5PathPickerModes.OpenFilesMode:
-            directory = os.path.expanduser(directory.split(";")[0])
-        else:
-            directory = os.path.expanduser(directory)
+        directory = (
+            os.path.expanduser(directory.split(";")[0])
+            if self.__mode == E5PathPickerModes.OpenFilesMode else
+            os.path.expanduser(directory)
+        )
         if not os.path.isabs(directory) and self.__defaultDirectory:
             directory = os.path.join(self.__defaultDirectory, directory)
         directory = QDir.fromNativeSeparators(directory)
@@ -683,7 +686,7 @@ class E5PathPicker(E5PathPickerBase):
         @param parent reference to the parent widget
         @type QWidget
         """
-        super(E5PathPicker, self).__init__(parent, useLineEdit=True)
+        super().__init__(parent, useLineEdit=True)
 
 
 class E5ComboPathPicker(E5PathPickerBase):
@@ -698,7 +701,7 @@ class E5ComboPathPicker(E5PathPickerBase):
         @param parent reference to the parent widget
         @type QWidget
         """
-        super(E5ComboPathPicker, self).__init__(parent, useLineEdit=False)
+        super().__init__(parent, useLineEdit=False)
     
     def getPathItems(self):
         """
