@@ -15,6 +15,7 @@ import sys
 import os
 import re
 import shutil
+import contextlib
 
 from PyQt5.QtCore import (
     QDir, QByteArray, QCoreApplication, QT_VERSION, QProcess, qVersion
@@ -121,10 +122,11 @@ def isKdeDesktop():
         os.environ.get("XDG_SESSION_DESKTOP", "").lower() or
         os.environ.get("DESKTOP_SESSION", "").lower()
     )
-    if desktop:
-        isKDE = "kde" in desktop or "plasma" in desktop
-    else:
-        isKDE = bool(os.environ.get("KDE_FULL_SESSION", ""))
+    isKDE = (
+        "kde" in desktop or "plasma" in desktop
+        if desktop else
+        bool(os.environ.get("KDE_FULL_SESSION", ""))
+    )
     
     return isKDE
 
@@ -146,10 +148,11 @@ def isGnomeDesktop():
         os.environ.get("XDG_SESSION_DESKTOP", "").lower() or
         os.environ.get("GDMSESSION", "").lower()
     )
-    if desktop:
-        isGnome = "gnome" in desktop
-    else:
-        isGnome = bool(os.environ.get("GNOME_DESKTOP_SESSION_ID", ""))
+    isGnome = (
+        "gnome" in desktop
+        if desktop else
+        bool(os.environ.get("GNOME_DESKTOP_SESSION_ID", ""))
+    )
     
     return isGnome
 
@@ -321,13 +324,11 @@ def getQtBinariesPath():
     
     # step 2: try the qt5_applications package
     if not path:
-        try:
+        with contextlib.suppress(ImportError):
             import qt5_applications
             path = os.path.join(os.path.dirname(qt5_applications.__file__),
                                 "Qt", "bin")
-        except ImportError:
-            # qt5-applications is not installed; just go to the next step
-            pass
+            # if qt5-applications is not installed just go to the next step
     
     # step 3: determine from used Python interpreter (designer is test object)
     if not path:
@@ -372,11 +373,8 @@ def versionToTuple(version, length=3):
     # step 1: extract suffix
     version = re.split(r"[^\d.]", version)[0]
     for part in version.split("."):
-        try:
+        with contextlib.suppress(ValueError):
             versionParts.append(int(part.strip()))
-        except ValueError:
-            # skip non-integer parts
-            pass
     versionParts.extend([0] * length)
     
     return tuple(versionParts[:length])
@@ -549,10 +547,11 @@ def getWebBrowserSupport():
                               "webBrowserSupport.py")
     proc = QProcess()
     proc.start(sys.executable, [scriptPath, qVersion()])
-    if proc.waitForFinished(10000):
-        variant = str(proc.readAllStandardOutput(), "utf-8", 'replace').strip()
-    else:
-        variant = "None"
+    variant = (
+        str(proc.readAllStandardOutput(), "utf-8", 'replace').strip()
+        if proc.waitForFinished(10000) else
+        "None"
+    )
     return variant
 #
 # eflag: noqa = M801
