@@ -11,7 +11,8 @@ class.
 import logging
 import os
 
-from PyQt5.QtCore import pyqtSlot, QObject
+from PyQt5.QtCore import pyqtSlot, QObject, QCoreApplication
+from PyQt5.QtWidgets import QInputDialog
 
 from E5Gui.E5Application import e5App
 
@@ -22,8 +23,6 @@ import Preferences
 SupportedBoards = {
     "esp": {
         "ids": [
-            (0x1A86, 0x7523),       # HL-340
-            (0x10C4, 0xEA60),       # CP210x
             (0x0403, 0x6001),       # M5Stack ESP32 device"),
             (0x0403, 0x6001),       # FT232/FT245 (XinaBox CW01, CW02)
             (0x0403, 0x6010),       # FT2232C/D/L/HL/Q (ESP-WROVER-KIT)
@@ -31,44 +30,80 @@ SupportedBoards = {
             (0x0403, 0x6014),       # FT232H
             (0x0403, 0x6015),       # Sparkfun ESP32
             (0x0403, 0x601C),       # FT4222H
+            (0x10C4, 0xEA60),       # CP210x
+            (0x1A86, 0x7523),       # HL-340
         ],
-        "description": "ESP8266, ESP32",
+        "description": "ESP32, ESP8266",
         "icon": "esp32Device",
         "port_description": "",
     },
     
     "circuitpython": {
         "ids": [
-            (0x2B04, 0xC00C),       # Particle Argon
-            (0x2B04, 0xC00D),       # Particle Boron
-            (0x2B04, 0xC00E),       # Particle Xenon
-            (0x239A, None),         # Any Adafruit Boards
+            (0x04D8, 0xEAD1),       # BH Dynamics DynOSSAT-EDU-EPS-v1.0
+            (0x04D8, 0xEAD2),       # BH Dynamics DynOSSAT-EDU-OBC-v1.0
+            (0x04D8, 0xEC44),       # maholli PyCubed
+            (0x04D8, 0xEC63),       # Kevin Neubauer CircuitBrains Basic
+            (0x04D8, 0xEC64),       # Kevin Neubauer CircuitBrains Deluxe
+            (0x04D8, 0xEC72),       # XinaBox CC03
+            (0x04D8, 0xEC75),       # XinaBox CS11
+            (0x04D8, 0xED5F),       # Itaca Innovation uChip CircuitPython
+            (0x04D8, 0xED94),       # maholli kicksat-sprite
+            (0x04D8, 0xEDB3),       # Capable Robot Programmable USB Hub
+            (0x04D8, 0xEDBE),       # maholli SAM32
+            (0x04D8, 0xEE8C),       # J&J Studios LLC datum-Distance
+            (0x04D8, 0xEE8D),       # J&J Studios LLC datum-IMU
+            (0x04D8, 0xEE8E),       # J&J Studios LLC datum-Light
+            (0x04D8, 0xEE8F),       # J&J Studios LLC datum-Weather
+            (0x054C, 0x0BC2),       # Sony Spresense
+            (0x1209, 0x2017),       # Benjamin Shockley Mini SAM M4
+            (0x1209, 0x3252),       # Targett Module Clip w/Wroom
+            (0x1209, 0x3253),       # Targett Module Clip w/Wrover
+            (0x1209, 0x4D43),       # Robotics Masters Robo HAT MM1 M4
+            (0x1209, 0x4DDD),       # ODT CP Sapling
+            (0x1209, 0x4DDE),       # ODT CP Sapling M0 w/ SPI Flash
+            (0x1209, 0x5BF0),       # Foosn Fomu
+            (0x1209, 0x805A),       # Electronic Cats BastBLE
+            (0x1209, 0xBAB0),       # Electronic Cats Bast WiFi
             (0x1209, 0xBAB1),       # Electronic Cats Meow Meow
             (0x1209, 0xBAB2),       # Electronic Cats CatWAN USBStick
             (0x1209, 0xBAB3),       # Electronic Cats Bast Pro Mini M0
             (0x1209, 0xBAB6),       # Electronic Cats Escornabot Makech
+            (0x1209, 0xBAB8),       # Electronic Cats NFC Copy Cat
+            (0x1209, 0xC051),       # Betrusted Simmel
+            (0x1209, 0xE3E3),       # StackRduino M0 PRO
+            (0x1209, 0xF500),       # Silicognition LLC M4-Shim
+            (0x1915, 0xB001),       # Makerdiary Pitaya Go
+            (0x1B4F, 0x0015),       # SparkFun RedBoard Turbo Board
+            (0x1B4F, 0x0016),       # SparkFun SAMD51 Thing+
+            (0x1B4F, 0x0017),       # SparkFun LUMIDrive Board
+            (0x1B4F, 0x5289),       # SparkFun SFE_nRF52840_Mini
             (0x1B4F, 0x8D22),       # SparkFun SAMD21 Mini Breakout
             (0x1B4F, 0x8D23),       # SparkFun SAMD21 Dev Breakout
-            (0x1209, 0x2017),       # Mini SAM M4
-            (0x1209, 0x7102),       # Mini SAM M0
-            (0x04D8, 0xEC72),       # XinaBox CC03
-            (0x04D8, 0xEC75),       # XinaBox CS11
-            (0x04D8, 0xED5E),       # XinaBox CW03
-            (0x3171, 0x0101),       # 8086.net Commander
-            (0x04D8, 0xED94),       # PyCubed
-            (0x04D8, 0xEDBE),       # SAM32
-            (0x1D50, 0x60E8),       # PewPew Game Console
-            (0x2886, 0x802D),       # Seeed Wio Terminal
-            (0x2886, 0x002F),       # Seeed XIAO
-            (0x1B4F, 0x0016),       # Sparkfun Thing Plus - SAMD51
-            (0x2341, 0x8057),       # Arduino Nano 33 IoT board
-            (0x04D8, 0xEAD1),       # DynOSSAT-EDU-EPS
-            (0x04D8, 0xEAD2),       # DynOSSAT-EDU-OBC
-            (0x1209, 0x4DDD),       # ODT CP Sapling M0
-            (0x1209, 0x4DDE),       # ODT CP Sapling M0 w/ SPI Flash
-            (0x054C, 0x0BC2),       # Spresense
+            (0x1B4F, 0x8D24),       # SparkFun Qwiic Micro
+            (0x1D50, 0x60E8),       # Radomir Dopieralski PewPew M4
+            (0x2341, 0x8053),       # Arduino MKR1300
+            (0x2341, 0x8057),       # Arduino Nano 33 IoT
+            (0x2341, 0x805A),       # Arduino Arduino_Nano_33_BLE
+            (0x2341, 0x824D),       # Arduino Zero
+            (0x2786, 0x9207),       # Switch Sc. BLE-SS dev board Multi Sensor
+            (0x2886, 0x002F),       # Seeed Seeeduino XIAO
+            (0x2886, 0x802D),       # Seeed Seeeduino Wio Terminal
+            (0x2886, 0xF001),       # Makerdiary nRF52840 M.2 Developer Kit
+            (0x2886, 0xF002),       # Makerdiary M60 Keyboard
+            (0x2B04, 0xC00C),       # Particle Argon
+            (0x2B04, 0xC00D),       # Particle Boron
+            (0x2B04, 0xC00E),       # Particle Xenon
+            (0x303A, 0x8007),       # LILYGO TTGO T8 ESP32-S2
+            (0x3171, 0x0101),       # 8086 Consultancy Commander
+            (0x31E2, 0x2001),       # BDMICRO LLC VINA-D21
+            (0x31E2, 0x2011),       # BDMICRO LLC VINA-D51
+            (0x32BD, 0x3001),       # Alorium Tech. AloriumTech Evo M51
+            (0x4097, 0x0001),       # TG-Boards Datalore IP M4
+            
+            (0x239A, None),         # Any Adafruit Boards
         ],
-        "description": "CircuitPython Board",
+        "description": "CircuitPython",
         "icon": "circuitPythonDevice",
         "port_description": "",
     },
@@ -101,6 +136,25 @@ SupportedBoards = {
         "icon": "micropython48",
         "port_description": "",
     },
+    
+    "rp2040": {
+        "ids": [
+            (0x2E8A, 0x0005),       # Raspberry Pi Pico
+        ],
+        "description": QCoreApplication.translate(
+            "MicroPythonDevice", "RP2040 based"),
+        "icon": "rp2040Device",
+        "port_description": "",
+    },
+    
+    "generic": {
+        # only manually configured devices use this
+        "ids": [],
+        "description": QCoreApplication.translate(
+            "MicroPythonDevice", "Generic Board"),
+        "icon": "micropython48",
+        "port_description": "",
+    },
 }
 
 IgnoredBoards = (
@@ -126,45 +180,84 @@ def getFoundDevices():
     """
     Function to check the serial ports for supported MicroPython devices.
     
-    @return tuple containing a list of tuples with the board type, a
-        description and the serial port it is connected at for known device
-        types and a list of tuples with VID, PID and description for unknown
-        devices
-    @rtype tuple of (list of tuples of (str, str, str), list of tuples of
-        (int, int, str)
+    @return tuple containing a list of tuples with the board type, the port
+        description, a description, the serial port it is connected at, the
+        VID and PID for known device types, a list of tuples with VID, PID
+        and description for unknown devices and a list of tuples with VID,
+        PID, description and port name for ports with missing VID or PID
+    @rtype tuple of (list of tuples of (str, str, str, str, int, int),
+        list of tuples of (int, int, str),
+        list of tuples of (int, int, str, str)
     """
     from PyQt5.QtSerialPort import QSerialPortInfo
     
     foundDevices = []
     unknownDevices = []
+    unknownPorts = []
+    
+    manualDevices = {}
+    for deviceDict in Preferences.getMicroPython("ManualDevices"):
+        manualDevices[(deviceDict["vid"], deviceDict["pid"])] = deviceDict
     
     availablePorts = QSerialPortInfo.availablePorts()
     for port in availablePorts:
         supported = False
         vid = port.vendorIdentifier()
         pid = port.productIdentifier()
+        
+        if not port.isValid():
+            # no device detected at port
+            continue
+        
         for board in SupportedBoards:
-            if ((vid, pid) in SupportedBoards[board]["ids"] or
-                    (vid, None) in SupportedBoards[board]["ids"]):
-                if board in ("bbc_microbit", "calliope"):
+            if (
+                (vid, pid) in SupportedBoards[board]["ids"] or
+                (vid, None) in SupportedBoards[board]["ids"]
+            ):
+                if (
+                    board in ("bbc_microbit", "calliope") and
+                    (port.description().strip() !=
+                     SupportedBoards[board]["port_description"])
+                ):
                     # both boards have the same VID and PID
                     # try to differentiate based on port description
-                    if (
-                        port.description().strip() !=
-                        SupportedBoards[board]["port_description"]
-                    ):
-                        continue
-                foundDevices.append(
-                    (board, SupportedBoards[board]["description"],
-                     port.portName()))
+                    continue
+                foundDevices.append((
+                    board,
+                    port.description(),
+                    SupportedBoards[board]["description"],
+                    port.portName(),
+                    vid,
+                    pid,
+                ))
                 supported = True
+        if not supported and (vid, pid) in manualDevices:
+            # check the locally added ones next
+            board = manualDevices[(vid, pid)]["type"]
+            foundDevices.append((
+                board,
+                port.description(),
+                SupportedBoards[board]["description"],
+                port.portName(),
+                vid,
+                pid,
+            ))
+            supported = True
         if not supported:
-            if vid and pid and (vid, pid) not in IgnoredBoards:
-                unknownDevices.append((vid, pid, port.description()))
-                logging.debug("Unknown device: (0x%04x:0x%04x %s)",
-                              vid, pid, port.description())
+            if vid and pid:
+                if (vid, pid) not in IgnoredBoards:
+                    unknownDevices.append((vid, pid, port.description()))
+                    logging.debug("Unknown device: (0x%04x:0x%04x %s)",
+                                  vid, pid, port.description())
+            else:
+                # either VID or PID or both not detected
+                desc = port.description()
+                if not desc:
+                    desc = QCoreApplication.translate("MicroPythonDevice",
+                                                      "Unknown Device")
+                unknownPorts.append((vid, pid, desc, port.portName()))
     
-    return foundDevices, unknownDevices
+    return foundDevices, unknownDevices, unknownPorts
 
 
 def getDeviceIcon(boardName, iconFormat=True):
@@ -179,11 +272,12 @@ def getDeviceIcon(boardName, iconFormat=True):
         a pixmap (iconFormat == False)
     @rtype QIcon or QPixmap
     """
-    if boardName in SupportedBoards:
-        iconName = SupportedBoards[boardName]["icon"]
-    else:
+    iconName = (
+        SupportedBoards[boardName]["icon"]
+        if boardName in SupportedBoards else
         # return a generic MicroPython icon
-        iconName = "micropython48"
+        "micropython48"
+    )
     
     if iconFormat:
         return UI.PixmapCache.getIcon(iconName)
@@ -191,7 +285,7 @@ def getDeviceIcon(boardName, iconFormat=True):
         return UI.PixmapCache.getPixmap(iconName)
 
 
-def getDevice(deviceType, microPythonWidget):
+def getDevice(deviceType, microPythonWidget, vid, pid):
     """
     Public method to instantiate a specific MicroPython device interface.
     
@@ -199,43 +293,66 @@ def getDevice(deviceType, microPythonWidget):
     @type str
     @param microPythonWidget reference to the main MicroPython widget
     @type MicroPythonWidget
+    @param vid vendor ID (only used for deviceType 'generic')
+    @type int
+    @param pid product ID (only used for deviceType 'generic')
+    @type int
     @return instantiated device interface
     @rtype MicroPythonDevice
     """
     if deviceType == "esp":
         from .EspDevices import EspDevice
-        return EspDevice(microPythonWidget)
+        return EspDevice(microPythonWidget, deviceType)
     elif deviceType == "circuitpython":
         from .CircuitPythonDevices import CircuitPythonDevice
-        return CircuitPythonDevice(microPythonWidget)
+        return CircuitPythonDevice(microPythonWidget, deviceType)
     elif deviceType in ("bbc_microbit", "calliope"):
         from .MicrobitDevices import MicrobitDevice
         return MicrobitDevice(microPythonWidget, deviceType)
     elif deviceType == "pyboard":
         from .PyBoardDevices import PyBoardDevice
-        return PyBoardDevice(microPythonWidget)
+        return PyBoardDevice(microPythonWidget, deviceType)
+    elif deviceType == "rp2040":
+        from .RP2040Devices import RP2040Device
+        return RP2040Device(microPythonWidget, deviceType)
+    elif deviceType == "generic":
+        from .GenericMicroPythonDevices import GenericMicroPythonDevice
+        return GenericMicroPythonDevice(microPythonWidget, deviceType,
+                                        vid, pid)
     else:
         # nothing specific requested
-        return MicroPythonDevice(microPythonWidget)
+        return MicroPythonDevice(microPythonWidget, deviceType)
 
 
 class MicroPythonDevice(QObject):
     """
     Base class for the more specific MicroPython devices.
     """
-    def __init__(self, microPythonWidget, parent=None):
+    def __init__(self, microPythonWidget, deviceType, parent=None):
         """
         Constructor
         
         @param microPythonWidget reference to the main MicroPython widget
         @type MicroPythonWidget
+        @param deviceType device type assigned to this device interface
+        @type str
         @param parent reference to the parent object
         @type QObject
         """
-        super(MicroPythonDevice, self).__init__(parent)
+        super().__init__(parent)
         
+        self._deviceType = deviceType
         self.microPython = microPythonWidget
     
+    def getDeviceType(self):
+        """
+        Public method to get the device type.
+        
+        @return type of the device
+        @rtype str
+        """
+        return self._deviceType
+    
     def setButtons(self):
         """
         Public method to enable the supported action buttons.
@@ -356,8 +473,33 @@ class MicroPythonDevice(QObject):
         @return workspace directory used for saving files
         @rtype str
         """
-        return (Preferences.getMultiProject("Workspace") or
-                os.path.expanduser("~"))
+        return (
+            Preferences.getMicroPython("MpyWorkspace") or
+            Preferences.getMultiProject("Workspace") or
+            os.path.expanduser("~")
+        )
+    
+    def selectDeviceDirectory(self, deviceDirectories):
+        """
+        Public method to select the device directory from a list of detected
+        ones.
+        
+        @param deviceDirectories list of directories to select from
+        @type list of str
+        @return selected directory or an empty string
+        @rtype str
+        """
+        deviceDirectory, ok = QInputDialog.getItem(
+            None,
+            self.tr("Select Device Directory"),
+            self.tr("Select the directory for the connected device:"),
+            [""] + deviceDirectories,
+            0, False)
+        if ok:
+            return deviceDirectory
+        else:
+            # user cancelled
+            return ""
     
     def sendCommands(self, commandsList):
         """
@@ -376,7 +518,7 @@ class MicroPythonDevice(QObject):
         commands = [c.encode("utf-8)") + b'\r' for c in commandsList]
         commands.append(b'\r')
         commands.append(b'\x04')
-        rawOff = [b'\x02']
+        rawOff = [b'\x02', b'\x02']
         commandSequence = rawOn + newLine + commands + rawOff
         self.microPython.commandsInterface().executeAsync(commandSequence)
     
@@ -396,6 +538,15 @@ class MicroPythonDevice(QObject):
         """
         pass
     
+    def hasFlashMenuEntry(self):
+        """
+        Public method to check, if the device has its own flash menu entry.
+        
+        @return flag indicating a specific flash menu entry
+        @rtype bool
+        """
+        return False
+    
     def hasTimeCommands(self):
         """
         Public method to check, if the device supports time commands.
@@ -452,3 +603,13 @@ class MicroPythonDevice(QObject):
         url = self.getFirmwareUrl()
         if url:
             e5App().getObject("UserInterface").launchHelpViewer(url)
+    
+    def getDownloadMenuEntries(self):
+        """
+        Public method to retrieve the entries for the downloads menu.
+        
+        @return list of tuples with menu text and URL to be opened for each
+            entry
+        @rtype list of tuple of (str, str)
+        """
+        return []
