@@ -1,13 +1,12 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2003 - 2020 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2003 - 2021 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
 Module implementing the file dialog wizard dialog.
 """
 
-
 import os
 
 from PyQt5.QtCore import pyqtSlot, QUrl
@@ -34,19 +33,17 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
         Constructor
         
         @param dialogVariant variant of the file dialog to be generated
-            (-1 = E5FileDialog, 0 = unknown, 4 = PyQt4, 5 = PyQt5)
+            (-1 = E5FileDialog, 0 = unknown, 5 = PyQt5)
         @type int
         @param parent parent widget
         @type QWidget
         """
-        super(FileDialogWizardDialog, self).__init__(parent)
+        super().__init__(parent)
         self.setupUi(self)
         
         self.eStartWithCompleter = E5FileCompleter(self.eStartWith)
         self.eWorkDirCompleter = E5DirCompleter(self.eWorkDir)
         
-        self.__dialogVariant = dialogVariant
-        
         self.__typeButtonsGroup = QButtonGroup(self)
         self.__typeButtonsGroup.setExclusive(True)
         self.__typeButtonsGroup.addButton(self.rOpenFile, 1)
@@ -71,11 +68,11 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
             self.pyqtComboBox.setCurrentIndex(0)
             self.pyqtComboBox.setEnabled(False)
         else:
-            self.pyqtComboBox.addItems(["PyQt5", "PyQt4"])
+            self.pyqtComboBox.addItems(["PyQt5", "PyQt6"])
             self.setWindowTitle(self.tr("QFileDialog Wizard"))
             if self.__dialogVariant == 5:
                 self.pyqtComboBox.setCurrentIndex(0)
-            elif self.__dialogVariant == 4:
+            elif self.__dialogVariant == 6:
                 self.pyqtComboBox.setCurrentIndex(1)
             else:
                 self.pyqtComboBox.setCurrentIndex(0)
@@ -90,7 +87,7 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
         self.cFilters.toggled[bool].connect(self.__toggleGroupsAndTest)
         
         self.bTest = self.buttonBox.addButton(
-            self.tr("Test"), QDialogButtonBox.ActionRole)
+            self.tr("Test"), QDialogButtonBox.ButtonRole.ActionRole)
         
         msh = self.minimumSizeHint()
         self.resize(max(self.width(), msh.width()), msh.height())
@@ -103,26 +100,28 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
         @return modified options (QFileDialog.Options)
         """
         if Globals.isLinuxPlatform():
-            options |= QFileDialog.DontUseNativeDialog
+            options |= QFileDialog.Option.DontUseNativeDialog
         return options
     
-    @pyqtSlot(str)
-    def on_pyqtComboBox_currentIndexChanged(self, txt):
+    @pyqtSlot(int)
+    def on_pyqtComboBox_currentIndexChanged(self, index):
         """
         Private slot to setup the dialog for the selected PyQt variant.
         
-        @param txt text of the selected combo box entry (string)
+        @param index index of the current item
+        @type int
         """
-        self.rfOpenFile.setEnabled(txt in ("eric", "PyQt4"))
-        self.rfOpenFiles.setEnabled(txt in ("eric", "PyQt4"))
-        self.rfSaveFile.setEnabled(txt in ("eric", "PyQt4"))
+        txt = self.pyqtComboBox.itemText(index)
+        self.rfOpenFile.setEnabled(txt == "eric")
+        self.rfOpenFiles.setEnabled(txt == "eric")
+        self.rfSaveFile.setEnabled(txt == "eric")
         
-        self.rOpenFileUrl.setEnabled(txt == "PyQt5")
-        self.rOpenFileUrls.setEnabled(txt == "PyQt5")
-        self.rSaveFileUrl.setEnabled(txt == "PyQt5")
-        self.rDirectoryUrl.setEnabled(txt == "PyQt5")
+        self.rOpenFileUrl.setEnabled(txt in ["PyQt5", "PyQt6"])
+        self.rOpenFileUrls.setEnabled(txt in ["PyQt5", "PyQt6"])
+        self.rSaveFileUrl.setEnabled(txt in ["PyQt5", "PyQt6"])
+        self.rDirectoryUrl.setEnabled(txt in ["PyQt5", "PyQt6"])
         
-        if txt == "PyQt5":
+        if txt in ["PyQt5", "PyQt6"]:
             if self.rfOpenFile.isChecked():
                 self.rOpenFile.setChecked(True)
             elif self.rfOpenFiles.isChecked():
@@ -143,8 +142,8 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
             self.__dialogVariant = -1
         elif txt == "PyQt5":
             self.__dialogVariant = 5
-        elif txt == "PyQt4":
-            self.__dialogVariant = 4
+        elif txt == "PyQt6":
+            self.__dialogVariant = 6
         else:
             # default is PyQt5
             self.__dialogVariant = 5
@@ -168,7 +167,8 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
         """
         if self.rOpenFile.isChecked() or self.rfOpenFile.isChecked():
             if not self.cSymlinks.isChecked():
-                options = QFileDialog.Options(QFileDialog.DontResolveSymlinks)
+                options = QFileDialog.Options(
+                    QFileDialog.Option.DontResolveSymlinks)
             else:
                 options = QFileDialog.Options()
             options = self.__adjustOptions(options)
@@ -181,7 +181,8 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                 options)
         elif self.rOpenFileUrl.isChecked():
             if not self.cSymlinks.isChecked():
-                options = QFileDialog.Options(QFileDialog.DontResolveSymlinks)
+                options = QFileDialog.Options(
+                    QFileDialog.Option.DontResolveSymlinks)
             else:
                 options = QFileDialog.Options()
             options = self.__adjustOptions(options)
@@ -206,7 +207,8 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                     self.schemesEdit.text().split())
         elif self.rOpenFiles.isChecked() or self.rfOpenFiles.isChecked():
             if not self.cSymlinks.isChecked():
-                options = QFileDialog.Options(QFileDialog.DontResolveSymlinks)
+                options = QFileDialog.Options(
+                    QFileDialog.Option.DontResolveSymlinks)
             else:
                 options = QFileDialog.Options()
             options = self.__adjustOptions(options)
@@ -219,7 +221,8 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                 options)
         elif self.rOpenFileUrls.isChecked():
             if not self.cSymlinks.isChecked():
-                options = QFileDialog.Options(QFileDialog.DontResolveSymlinks)
+                options = QFileDialog.Options(
+                    QFileDialog.Option.DontResolveSymlinks)
             else:
                 options = QFileDialog.Options()
             options = self.__adjustOptions(options)
@@ -244,7 +247,8 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                     self.schemesEdit.text().split())
         elif self.rSaveFile.isChecked() or self.rfSaveFile.isChecked():
             if not self.cSymlinks.isChecked():
-                options = QFileDialog.Options(QFileDialog.DontResolveSymlinks)
+                options = QFileDialog.Options(
+                    QFileDialog.Option.DontResolveSymlinks)
             else:
                 options = QFileDialog.Options()
             options = self.__adjustOptions(options)
@@ -257,7 +261,8 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                 options)
         elif self.rSaveFileUrl.isChecked():
             if not self.cSymlinks.isChecked():
-                options = QFileDialog.Options(QFileDialog.DontResolveSymlinks)
+                options = QFileDialog.Options(
+                    QFileDialog.Option.DontResolveSymlinks)
             else:
                 options = QFileDialog.Options()
             options = self.__adjustOptions(options)
@@ -283,9 +288,10 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
         elif self.rDirectory.isChecked():
             options = QFileDialog.Options()
             if not self.cSymlinks.isChecked():
-                options |= QFileDialog.Options(QFileDialog.DontResolveSymlinks)
+                options |= QFileDialog.Options(
+                    QFileDialog.Option.DontResolveSymlinks)
             if self.cDirOnly.isChecked():
-                options |= QFileDialog.Options(QFileDialog.ShowDirsOnly)
+                options |= QFileDialog.Options(QFileDialog.Option.ShowDirsOnly)
             else:
                 options |= QFileDialog.Options(QFileDialog.Option(0))
             options = self.__adjustOptions(options)
@@ -297,9 +303,10 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
         elif self.rDirectoryUrl.isChecked():
             options = QFileDialog.Options()
             if not self.cSymlinks.isChecked():
-                options |= QFileDialog.Options(QFileDialog.DontResolveSymlinks)
+                options |= QFileDialog.Options(
+                    QFileDialog.Option.DontResolveSymlinks)
             if self.cDirOnly.isChecked():
-                options |= QFileDialog.Options(QFileDialog.ShowDirsOnly)
+                options |= QFileDialog.Options(QFileDialog.Option.ShowDirsOnly)
             else:
                 options |= QFileDialog.Options(QFileDialog.Option(0))
             options = self.__adjustOptions(options)
@@ -349,8 +356,9 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
         @param checkedId id of the clicked button (integer)
         """
         enable = (
-            (self.__dialogVariant in (-1, 4) and checkedId in [11, 12, 13]) or
-            (self.__dialogVariant == 5 and checkedId in [1, 2, 3, 21, 22, 23])
+            (self.__dialogVariant in (-1, ) and checkedId in [11, 12, 13]) or
+            (self.__dialogVariant in (5, 6) and
+             checkedId in [1, 2, 3, 21, 22, 23])
         )
         
         self.lInitialFilter.setEnabled(enable)
@@ -364,7 +372,7 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
     
     def getCode(self, indLevel, indString):
         """
-        Public method to get the source code for Qt4 and Qt5.
+        Public method to get the source code for Qt5.
         
         @param indLevel indentation level (int)
         @param indString string used for indentation (space or tab) (string)
@@ -402,10 +410,10 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
         filterVariable = self.eFilterVariable.text()
         if not filterVariable:
             if (
-                (self.__dialogVariant in (-1, 4) and
+                (self.__dialogVariant in (-1, ) and
                  self.__typeButtonsGroup.checkedButton() in [
                     self.rfOpenFile, self.rfOpenFiles, self.rfSaveFile]) or
-                (self.__dialogVariant == 5 and
+                (self.__dialogVariant in (5, 6) and
                  self.__typeButtonsGroup.checkedButton() in [
                     self.rOpenFile, self.rOpenFiles, self.rSaveFile])
             ):
@@ -417,8 +425,10 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
         
         if self.__dialogVariant == -1:
             dialogType = "E5FileDialog"
+            optionStr = ""
         else:
             dialogType = "QFileDialog"
+            optionStr = ".Option"
         
         code = '{0}{1} = {2}.'.format(nameVariable, filterVariable, dialogType)
         if (
@@ -467,7 +477,7 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                 else:
                     fmt = 'self.tr("{0}")'
                 code += fmt.format(self.eFilters.text())
-            if self.rfOpenFile.isChecked() or self.__dialogVariant == 5:
+            if self.rfOpenFile.isChecked() or self.__dialogVariant in (5, 6):
                 if self.eInitialFilter.text() == "":
                     initialFilter = "None"
                 else:
@@ -478,8 +488,10 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                     initialFilter = fmt.format(self.eInitialFilter.text())
                 code += ',{0}{1}{2}'.format(os.linesep, istring, initialFilter)
             if not self.cSymlinks.isChecked():
-                code += ',{0}{1}{2}.Options({2}.DontResolveSymlinks)'.format(
-                    os.linesep, istring, dialogType)
+                code += (
+                    ',{0}{1}{2}.Options({2}{3}.DontResolveSymlinks)'
+                    .format(os.linesep, istring, dialogType, optionStr)
+                )
             if self.rOpenFileUrl.isChecked() and bool(self.schemesEdit.text()):
                 code += ',{0}{1}{2}'.format(
                     os.linesep, istring, self.__prepareSchemesList())
@@ -530,7 +542,7 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                 else:
                     fmt = 'self.tr("{0}")'
                 code += fmt.format(self.eFilters.text())
-            if self.rfOpenFiles.isChecked() or self.__dialogVariant == 5:
+            if self.rfOpenFiles.isChecked() or self.__dialogVariant in (5, 6):
                 if self.eInitialFilter.text() == "":
                     initialFilter = "None"
                 else:
@@ -541,8 +553,10 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                     initialFilter = fmt.format(self.eInitialFilter.text())
                 code += ',{0}{1}{2}'.format(os.linesep, istring, initialFilter)
             if not self.cSymlinks.isChecked():
-                code += ',{0}{1}{2}.Options({2}.DontResolveSymlinks)'.format(
-                    os.linesep, istring, dialogType)
+                code += (
+                    ',{0}{1}{2}.Options({2}{3}.DontResolveSymlinks)'
+                    .format(os.linesep, istring, dialogType, optionStr)
+                )
             if (
                 self.rOpenFileUrls.isChecked() and
                 bool(self.schemesEdit.text())
@@ -596,7 +610,7 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                 else:
                     fmt = 'self.tr("{0}")'
                 code += fmt.format(self.eFilters.text())
-            if self.rfSaveFile.isChecked() or self.__dialogVariant == 5:
+            if self.rfSaveFile.isChecked() or self.__dialogVariant in (5, 6):
                 if self.eInitialFilter.text() == "":
                     initialFilter = "None"
                 else:
@@ -613,14 +627,16 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
                 code += ',{0}{1}{2}.Options('.format(
                     os.linesep, istring, dialogType)
                 if not self.cSymlinks.isChecked():
-                    code += '{0}.DontResolveSymlinks'.format(dialogType)
+                    code += '{0}{1}.DontResolveSymlinks'.format(
+                        dialogType, optionStr)
                 if (
                     (not self.cSymlinks.isChecked()) and
                     (not self.cConfirmOverwrite.isChecked())
                 ):
                     code += ' | '
                 if not self.cConfirmOverwrite.isChecked():
-                    code += '{0}.DontConfirmOverwrite'.format(dialogType)
+                    code += '{0}{1}.DontConfirmOverwrite'.format(
+                        dialogType, optionStr)
                 code += ')'
             if (
                 self.rSaveFileUrl.isChecked() and
@@ -664,9 +680,11 @@ class FileDialogWizardDialog(QDialog, Ui_FileDialogWizardDialog):
             code += ',{0}{1}{2}.Options('.format(os.linesep, istring,
                                                  dialogType)
             if not self.cSymlinks.isChecked():
-                code += '{0}.DontResolveSymlinks | '.format(dialogType)
+                code += '{0}{1}.DontResolveSymlinks | '.format(
+                    dialogType, optionStr)
             if self.cDirOnly.isChecked():
-                code += '{0}.ShowDirsOnly'.format(dialogType)
+                code += '{0}{1}.ShowDirsOnly'.format(
+                    dialogType, optionStr)
             else:
                 code += '{0}.Option(0)'.format(dialogType)
             code += ')'
