@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2020 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2021 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -8,6 +8,7 @@ Module implementing a class used to display the Sources part of the project.
 """
 
 import os
+import contextlib
 
 from PyQt5.QtCore import pyqtSignal
 from PyQt5.QtWidgets import QDialog, QInputDialog, QMenu
@@ -580,7 +581,7 @@ class ProjectSourcesBrowser(ProjectBaseBrowser):
         if not self.project.isOpen():
             return
         
-        try:
+        with contextlib.suppress(Exception):
             categories = self.getSelectedItemsCountCategorized(
                 [ProjectBrowserFileItem, BrowserClassItem,
                  BrowserMethodItem, ProjectBrowserSimpleDirectoryItem,
@@ -669,8 +670,6 @@ class ProjectSourcesBrowser(ProjectBaseBrowser):
                         self.backMenu.popup(self.mapToGlobal(coord))
                 else:
                     self.backMenu.popup(self.mapToGlobal(coord))
-        except Exception:           # secok
-            pass
         
     def __showContextMenu(self):
         """
@@ -814,7 +813,7 @@ class ProjectSourcesBrowser(ProjectBaseBrowser):
             dn = dn[1:]
         from .NewPythonPackageDialog import NewPythonPackageDialog
         dlg = NewPythonPackageDialog(dn, self)
-        if dlg.exec() == QDialog.Accepted:
+        if dlg.exec() == QDialog.DialogCode.Accepted:
             packageName = dlg.getData()
             nameParts = packageName.split(".")
             packagePath = self.project.ppath
@@ -915,7 +914,7 @@ class ProjectSourcesBrowser(ProjectBaseBrowser):
                 "Do you really want to delete these files from the project?"),
             files)
         
-        if dlg.exec() == QDialog.Accepted:
+        if dlg.exec() == QDialog.DialogCode.Accepted:
             for fn2, fn in zip(fullNames, files):
                 self.closeSourceWindow.emit(fn2)
                 self.project.deleteFile(fn)
