@@ -50,7 +50,7 @@ class TaskViewer(QTreeWidget):
         @param parent the parent (QWidget)
         @param project reference to the project object
         """
-        super(TaskViewer, self).__init__(parent)
+        super().__init__(parent)
         
         self.setSortingEnabled(True)
         self.setExpandsOnDoubleClick(False)
@@ -64,7 +64,7 @@ class TaskViewer(QTreeWidget):
             1, UI.PixmapCache.getIcon("taskPriority"))
         self.setHeaderItem(self.__headerItem)
         
-        self.header().setSortIndicator(2, Qt.AscendingOrder)
+        self.header().setSortIndicator(2, Qt.SortOrder.AscendingOrder)
         self.__resizeColumns()
         
         self.tasks = []
@@ -163,7 +163,7 @@ class TaskViewer(QTreeWidget):
         
         self.__activating = False
         
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         self.customContextMenuRequested.connect(self.__showContextMenu)
         self.itemActivated.connect(self.__taskItemActivated)
         
@@ -208,7 +208,7 @@ class TaskViewer(QTreeWidget):
         """
         Private method to resize the list columns.
         """
-        self.header().resizeSections(QHeaderView.ResizeToContents)
+        self.header().resizeSections(QHeaderView.ResizeMode.ResizeToContents)
         self.header().setStretchLastSection(True)
     
     def findParentTask(self, parentUid):
@@ -482,7 +482,7 @@ class TaskViewer(QTreeWidget):
         ro = task.getFilename() != ""
         if ro:
             dlg.setReadOnly()
-        if dlg.exec() == QDialog.Accepted and not ro:
+        if dlg.exec() == QDialog.DialogCode.Accepted and not ro:
             summary, priority, completed, isProjectTask, description = (
                 dlg.getData()
             )
@@ -499,7 +499,7 @@ class TaskViewer(QTreeWidget):
         """
         from .TaskPropertiesDialog import TaskPropertiesDialog
         dlg = TaskPropertiesDialog(None, self, self.projectOpen)
-        if dlg.exec() == QDialog.Accepted:
+        if dlg.exec() == QDialog.DialogCode.Accepted:
             summary, priority, completed, isProjectTask, description = (
                 dlg.getData()
             )
@@ -516,7 +516,7 @@ class TaskViewer(QTreeWidget):
         from .TaskPropertiesDialog import TaskPropertiesDialog
         dlg = TaskPropertiesDialog(None, self, self.projectOpen)
         dlg.setSubTaskMode(projectTask)
-        if dlg.exec() == QDialog.Accepted:
+        if dlg.exec() == QDialog.DialogCode.Accepted:
             summary, priority, completed, isProjectTask, description = (
                 dlg.getData()
             )
@@ -553,7 +553,7 @@ class TaskViewer(QTreeWidget):
         ci = self.currentItem()
         if ci:
             ind = self.indexFromItem(ci, self.currentColumn())
-            self.scrollTo(ind, QAbstractItemView.PositionAtCenter)
+            self.scrollTo(ind, QAbstractItemView.ScrollHint.PositionAtCenter)
     
     def __copyTask(self):
         """
@@ -631,7 +631,7 @@ class TaskViewer(QTreeWidget):
         ci = self.currentItem()
         if ci:
             ind = self.indexFromItem(ci, self.currentColumn())
-            self.scrollTo(ind, QAbstractItemView.PositionAtCenter)
+            self.scrollTo(ind, QAbstractItemView.ScrollHint.PositionAtCenter)
     
     def __goToTask(self):
         """
@@ -678,7 +678,7 @@ class TaskViewer(QTreeWidget):
         """
         from .TaskFilterConfigDialog import TaskFilterConfigDialog
         dlg = TaskFilterConfigDialog(self.taskFilter)
-        if dlg.exec() == QDialog.Accepted:
+        if dlg.exec() == QDialog.DialogCode.Accepted:
             dlg.configureTaskFilter(self.taskFilter)
             self.__refreshDisplay()
 
@@ -691,7 +691,7 @@ class TaskViewer(QTreeWidget):
             self.tr("Scan Filter Patterns"),
             self.tr("Enter filename patterns of files"
                     " to be excluded separated by a comma:"),
-            QLineEdit.Normal,
+            QLineEdit.EchoMode.Normal,
             self.__projectTasksScanFilter)
         if ok:
             self.__projectTasksScanFilter = scanFilter
@@ -732,10 +732,9 @@ class TaskViewer(QTreeWidget):
                 self.tr("Abort"), 0, len(files), self.tr("%v/%m Files"))
             progress.setMinimumDuration(0)
             progress.setWindowTitle(self.tr("Tasks"))
-            count = 0
             
             ppath = self.project.getProjectPath()
-            for file in files:
+            for count, file in enumerate(files):
                 progress.setLabelText(
                     self.tr("Extracting project tasks...\n{0}").format(file))
                 progress.setValue(count)
@@ -754,9 +753,7 @@ class TaskViewer(QTreeWidget):
                     continue
                 
                 # now search tasks and record them
-                lineIndex = 0
-                for line in lines:
-                    lineIndex += 1
+                for lineIndex, line in enumerate(lines, start=1):
                     shouldBreak = False
                     
                     if line.endswith("__NO-TASK__"):
@@ -773,8 +770,6 @@ class TaskViewer(QTreeWidget):
                                 break
                         if shouldBreak:
                             break
-                
-                count += 1
             
             progress.setValue(len(files))
     
@@ -833,7 +828,7 @@ class ProjectTaskExtractionThread(QThread):
         
         @param parent reference to the parent object (QObject)
         """
-        super(ProjectTaskExtractionThread, self).__init__()
+        super().__init__()
         
         self.__lock = threading.Lock()
         self.__interrupt = False
@@ -862,7 +857,7 @@ class ProjectTaskExtractionThread(QThread):
                 self.__markers[markerType] = markers[markerType][:]
             
             if not self.isRunning():
-                self.start(QThread.LowPriority)
+                self.start(QThread.Priority.LowPriority)
     
     def run(self):
         """
@@ -886,12 +881,10 @@ class ProjectTaskExtractionThread(QThread):
                 continue
             
             # now search tasks and record them
-            lineIndex = 0
-            for line in lines:
+            for lineIndex, line in enumerate(lines, start=1):
                 if self.__interrupt:
                     break
                 
-                lineIndex += 1
                 found = False
                 
                 if line.endswith("__NO-TASK__"):
