@@ -89,7 +89,7 @@ class Browser(QTreeView):
         
         @param parent parent widget (QWidget)
         """
-        super(Browser, self).__init__(parent)
+        super().__init__(parent)
         
         self.setWindowTitle(QCoreApplication.translate('Browser',
                                                        'File-Browser'))
@@ -104,7 +104,7 @@ class Browser(QTreeView):
         
         self._activating = False
         
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         self.customContextMenuRequested.connect(self._contextMenuRequested)
         self.activated.connect(self._openItem)
         self.expanded.connect(self._resizeColumns)
@@ -167,14 +167,16 @@ class Browser(QTreeView):
         self.setAlternatingRowColors(True)
         
         header = self.header()
-        header.setSortIndicator(0, Qt.AscendingOrder)
+        header.setSortIndicator(0, Qt.SortOrder.AscendingOrder)
         header.setSortIndicatorShown(True)
         header.setSectionsClickable(True)
         
         self.setSortingEnabled(True)
         
-        self.setSelectionMode(QAbstractItemView.ExtendedSelection)
-        self.setSelectionBehavior(QAbstractItemView.SelectRows)
+        self.setSelectionMode(
+            QAbstractItemView.SelectionMode.ExtendedSelection)
+        self.setSelectionBehavior(
+            QAbstractItemView.SelectionBehavior.SelectRows)
         
         self.header().setStretchLastSection(True)
         self.headerSize0 = 0
@@ -382,8 +384,8 @@ class Browser(QTreeView):
             if index.isValid():
                 self.setCurrentIndex(index)
                 flags = QItemSelectionModel.SelectionFlags(
-                    QItemSelectionModel.ClearAndSelect |
-                    QItemSelectionModel.Rows)
+                    QItemSelectionModel.SelectionFlag.ClearAndSelect |
+                    QItemSelectionModel.SelectionFlag.Rows)
                 self.selectionModel().select(index, flags)
                 
                 itm = self.model().item(index)
@@ -464,7 +466,12 @@ class Browser(QTreeView):
             self._activating = True
             for itm in itmList:
                 if isinstance(itm, BrowserFileItem):
-                    if itm.isPython3File():
+                    if (
+                        itm.isPython3File() or
+                        itm.isIdlFile() or
+                        itm.isProtobufFile() or
+                        itm.isResourcesFile()
+                    ):
                         self.sourceFile[str].emit(itm.fileName())
                     elif itm.isRubyFile():
                         self.sourceFile[str, int, str].emit(
@@ -483,16 +490,11 @@ class Browser(QTreeView):
                         self.projectFile.emit(itm.fileName())
                     elif itm.isMultiProjectFile():
                         self.multiProjectFile.emit(itm.fileName())
-                    elif itm.isIdlFile():
-                        self.sourceFile[str].emit(itm.fileName())
-                    elif itm.isProtobufFile():
-                        self.sourceFile[str].emit(itm.fileName())
-                    elif itm.isResourcesFile():
-                        self.sourceFile[str].emit(itm.fileName())
                     elif itm.isSvgFile():
                         self.svgFile.emit(itm.fileName())
                     elif itm.isPixmapFile():
                         self.pixmapFile.emit(itm.fileName())
+                    # TODO: add entry for eric graphics files
                     else:
                         if Utilities.MimeTypes.isTextFile(itm.fileName()):
                             self.sourceFile[str].emit(itm.fileName())
@@ -578,9 +580,8 @@ class Browser(QTreeView):
         itmList = self.getSelectedItems([BrowserFileItem])
         
         for itm in itmList:
-            if isinstance(itm, BrowserFileItem):
-                if itm.isPixmapFile():
-                    self.pixmapEditFile.emit(itm.fileName())
+            if isinstance(itm, BrowserFileItem) and itm.isPixmapFile():
+                self.pixmapEditFile.emit(itm.fileName())
         
     def _openHexEditor(self):
         """
@@ -718,11 +719,9 @@ class Browser(QTreeView):
         """
         if filterList is None:
             filterList = self.selectedItemsFilter
-        for typ in filterList:
-            if isinstance(itm, typ):
-                return True
-        return False
         
+        return any(isinstance(itm, typ) for typ in filterList)
+    
     def getSelectedItems(self, filterList=None):
         """
         Public method to get the selected items.
@@ -801,10 +800,11 @@ class Browser(QTreeView):
         if self.model().rowCount() == 0:
             return
         
-        if self.currentIndex().isValid():
-            startIndex = self.currentIndex()
-        else:
-            startIndex = self.model().index(0, 0)
+        startIndex = (
+            self.currentIndex()
+            if self.currentIndex().isValid() else
+            self.model().index(0, 0)
+        )
         
         keyboardSearchTimeWasValid = self._keyboardSearchTimer.isValid()
         keyboardSearchTimeElapsed = self._keyboardSearchTimer.restart()
@@ -862,7 +862,7 @@ class Browser(QTreeView):
                 self,
                 self.tr("New Directory"),
                 self.tr("Name for new directory:"),
-                QLineEdit.Normal)
+                QLineEdit.EchoMode.Normal)
             if ok and bool(newName):
                 dirpath = os.path.join(dname, newName)
                 if os.path.exists(dirpath):
@@ -895,7 +895,7 @@ class Browser(QTreeView):
                 self,
                 self.tr("New File"),
                 self.tr("Name for new file:"),
-                QLineEdit.Normal)
+                QLineEdit.EchoMode.Normal)
             if ok and bool(fname):
                 filepath = os.path.join(dname, fname)
                 if os.path.exists(filepath):
@@ -907,8 +907,8 @@ class Browser(QTreeView):
                         .format(fname))
                 else:
                     try:
-                        f = open(filepath, "w")
-                        f.close()
+                        with open(filepath, "w"):
+                            pass
                     except OSError as err:
                         E5MessageBox.critical(
                             self,
@@ -938,7 +938,7 @@ class Browser(QTreeView):
         @type str
         """
         try:
-            from ThirdParty.Send2Trash.send2trash import send2trash as s2t
+            from send2trash import send2trash as s2t
             trashMsg = self.tr("Do you really want to move this file to the"
                                " trash?")
         except ImportError:
@@ -953,7 +953,7 @@ class Browser(QTreeView):
             self.tr("Delete File"),
             trashMsg,
             [fn])
-        if dlg.exec() == QDialog.Accepted:
+        if dlg.exec() == QDialog.DialogCode.Accepted:
             try:
                 s2t(fn)
             except OSError as err:
@@ -974,7 +974,7 @@ class Browser(QTreeView):
         @type str
         """
         try:
-            from ThirdParty.Send2Trash.send2trash import send2trash
+            from send2trash import send2trash
             s2tAvailable = True
             trashMsg = self.tr("Do you really want to move this directory to"
                                " the trash?")
@@ -990,7 +990,7 @@ class Browser(QTreeView):
             self.tr("Delete Directory"),
             trashMsg,
             [dn])
-        if dlg.exec() == QDialog.Accepted:
+        if dlg.exec() == QDialog.DialogCode.Accepted:
             try:
                 if s2tAvailable:
                     send2trash(dn)
@@ -1019,7 +1019,7 @@ class Browser(QTreeView):
             fileNames.append(itm.fileName())
         
         try:
-            from ThirdParty.Send2Trash.send2trash import send2trash as s2t
+            from send2trash import send2trash as s2t
             trashMsg = self.tr("Do you really want to move these files to the"
                                " trash?")
         except ImportError:
@@ -1035,7 +1035,7 @@ class Browser(QTreeView):
             trashMsg,
             sorted(fileNames)
         )
-        if dlg.exec() == QDialog.Accepted:
+        if dlg.exec() == QDialog.DialogCode.Accepted:
             for fn in fileNames:
                 try:
                     s2t(fn)
