@@ -1,16 +1,16 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2006 - 2020 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2006 - 2021 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
 Module implementing the browser model.
 """
 
-
 import os
 import fnmatch
 import json
+import contextlib
 
 from PyQt5.QtCore import (
     QDir, QModelIndex, QAbstractItemModel, QFileSystemWatcher, Qt, QProcess,
@@ -45,10 +45,10 @@ class BrowserModel(QAbstractItemModel):
         Constructor
         
         @param parent reference to parent object (QObject)
-        @keyparam nopopulate flag indicating to not populate the model
+        @param nopopulate flag indicating to not populate the model
             (boolean)
         """
-        super(BrowserModel, self).__init__(parent)
+        super().__init__(parent)
         
         self.progDir = None
         
@@ -77,10 +77,8 @@ class BrowserModel(QAbstractItemModel):
         if parent is None:
             parent = QModelIndex()
         
-        if parent.isValid():
-            item = parent.internalPointer()
-        else:
-            item = self.rootItem
+        item = (parent.internalPointer() if parent.isValid()
+                else self.rootItem)
         
         return item.columnCount() + 1
     
@@ -95,7 +93,7 @@ class BrowserModel(QAbstractItemModel):
         if not index.isValid():
             return None
         
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             item = index.internalPointer()
             if index.column() < item.columnCount():
                 return item.data(index.column())
@@ -106,10 +104,10 @@ class BrowserModel(QAbstractItemModel):
                 # This is for the case where an item under a multi-column
                 # parent doesn't have a value for all the columns
                 return ""
-        elif role == Qt.DecorationRole:
+        elif role == Qt.ItemDataRole.DecorationRole:
             if index.column() == 0:
                 return index.internalPointer().getIcon()
-        elif role == Qt.FontRole:
+        elif role == Qt.ItemDataRole.FontRole:
             item = index.internalPointer()
             if item.isSymlink():
                 font = QFont(QApplication.font("QTreeView"))
@@ -126,11 +124,12 @@ class BrowserModel(QAbstractItemModel):
         @return requested flags (Qt.ItemFlags)
         """
         if not index.isValid():
-            return Qt.ItemIsEnabled
+            return Qt.ItemFlag.ItemIsEnabled
         
-        return Qt.ItemIsEnabled | Qt.ItemIsSelectable
+        return Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemIsSelectable
     
-    def headerData(self, section, orientation, role=Qt.DisplayRole):
+    def headerData(self, section, orientation,
+                   role=Qt.ItemDataRole.DisplayRole):
         """
         Public method to get the header data.
         
@@ -139,7 +138,10 @@ class BrowserModel(QAbstractItemModel):
         @param role role of data (Qt.ItemDataRole)
         @return requested header data
         """
-        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
+        if (
+            orientation == Qt.Orientation.Horizontal and
+            role == Qt.ItemDataRole.DisplayRole
+        ):
             if section >= self.rootItem.columnCount():
                 return ""
             else:
@@ -170,10 +172,11 @@ class BrowserModel(QAbstractItemModel):
         ):
             return QModelIndex()
         
-        if not parent.isValid():
-            parentItem = self.rootItem
-        else:
-            parentItem = parent.internalPointer()
+        parentItem = (
+            parent.internalPointer()
+            if parent.isValid() else
+            self.rootItem
+        )
         
         try:
             if not parentItem.isPopulated():
@@ -318,12 +321,8 @@ class BrowserModel(QAbstractItemModel):
             # just ignore the situation we don't have a reference to the item
             return
         
-        if Preferences.getUI("BrowsersListHiddenFiles"):
-            dirFilter = QDir.Filters(
-                QDir.AllEntries | QDir.Hidden | QDir.NoDotAndDotDot)
-        else:
-            dirFilter = QDir.Filters(
-                QDir.AllEntries | QDir.NoDot | QDir.NoDotDot)
+        dirFilter = QDir.Filters(QDir.Filter.AllEntries |
+                                 QDir.Filter.NoDotAndDotDot)
         
         for itm in self.watchedItems[path]:
             oldCnt = itm.childCount()
@@ -348,15 +347,16 @@ class BrowserModel(QAbstractItemModel):
                 cnt = itm.childCount()
                 self.beginInsertRows(
                     self.createIndex(itm.row(), 0, itm), cnt, cnt)
-                if f.isDir():
-                    node = BrowserDirectoryItem(
+                node = (
+                    BrowserDirectoryItem(
                         itm,
                         Utilities.toNativeSeparators(f.absoluteFilePath()),
                         False)
-                else:
-                    node = BrowserFileItem(
+                    if f.isDir() else
+                    BrowserFileItem(
                         itm,
                         Utilities.toNativeSeparators(f.absoluteFilePath()))
+                )
                 self._addItem(node, itm)
                 self.endInsertRows()
             
@@ -512,10 +512,11 @@ class BrowserModel(QAbstractItemModel):
         if parent is None:
             parent = QModelIndex()
         
-        if not parent.isValid():
-            parentItem = self.rootItem
-        else:
-            parentItem = parent.internalPointer()
+        parentItem = (
+            parent.internalPointer()
+            if parent.isValid() else
+            self.rootItem
+        )
         
         cnt = parentItem.childCount()
         self.beginInsertRows(parent, cnt, cnt)
@@ -553,12 +554,9 @@ class BrowserModel(QAbstractItemModel):
         
         qdir = QDir(parentItem.dirName())
         
-        if Preferences.getUI("BrowsersListHiddenFiles"):
-            dirFilter = QDir.Filters(
-                QDir.AllEntries | QDir.Hidden | QDir.NoDotAndDotDot)
-        else:
-            dirFilter = QDir.Filters(
-                QDir.AllEntries | QDir.NoDot | QDir.NoDotDot)
+        dirFilter = QDir.Filters(
+            QDir.Filter.AllEntries |
+            QDir.Filter.NoDotAndDotDot)
         entryInfoList = qdir.entryInfoList(dirFilter)
         if len(entryInfoList) > 0:
             if repopulate:
@@ -609,10 +607,11 @@ class BrowserModel(QAbstractItemModel):
                             self.createIndex(parentItem.row(), 0, parentItem),
                             0, len(syspath) - 1)
                     for p in syspath:
-                        if os.path.isdir(p):
-                            node = BrowserDirectoryItem(parentItem, p)
-                        else:
-                            node = BrowserFileItem(parentItem, p)
+                        node = (
+                            BrowserDirectoryItem(parentItem, p)
+                            if os.path.isdir(p) else
+                            BrowserFileItem(parentItem, p)
+                        )
                         self._addItem(node, parentItem)
                     if repopulate:
                         self.endInsertRows()
@@ -632,28 +631,33 @@ class BrowserModel(QAbstractItemModel):
         try:
             dictionary = Utilities.ClassBrowsers.readmodule(
                 moduleName, [parentItem.dirName()],
-                parentItem.isPython3File())
+                parentItem.isPython3File() or parentItem.isCythonFile())
         except ImportError:
             return
         
         keys = list(dictionary.keys())
         if len(keys) > 0:
             if repopulate:
+                last = len(keys) - 1
+                if (
+                    "@@Coding@@" in keys and
+                    not Preferences.getUI("BrowserShowCoding")
+                ):
+                    last -= 1
                 self.beginInsertRows(
                     self.createIndex(parentItem.row(), 0, parentItem),
-                    0, len(keys) - 1)
+                    0, last)
+            
             for key in keys:
                 if key.startswith("@@"):
                     # special treatment done later
                     continue
                 cl = dictionary[key]
-                try:
+                with contextlib.suppress(AttributeError):
                     if cl.module == moduleName:
                         node = BrowserClassItem(parentItem, cl, fileName)
                         self._addItem(node, parentItem)
-                except AttributeError:
-                    pass
-            if "@@Coding@@" in keys:
+            if "@@Coding@@" in keys and Preferences.getUI("BrowserShowCoding"):
                 node = BrowserCodingItem(
                     parentItem,
                     QCoreApplication.translate("BrowserModel", "Coding: {0}")
@@ -691,8 +695,10 @@ class BrowserModel(QAbstractItemModel):
                                 linenos,
                                 isModule=False)
                             self._addItem(mn_node, m_node)
+            
             if repopulate:
                 self.endInsertRows()
+        
         parentItem._populated = True
         if (
             parentItem.type_ == BrowserItemFile and
@@ -855,7 +861,7 @@ class BrowserModel(QAbstractItemModel):
                 self.endInsertRows()
 
 
-class BrowserItem(object):
+class BrowserItem:
     """
     Class implementing the data structure for browser items.
     """
@@ -1039,12 +1045,7 @@ class BrowserDirectoryItem(BrowserItem):
         @param full flag indicating full pathname should be displayed (boolean)
         """
         self._dirName = os.path.abspath(dinfo)
-        
-        if full:
-            dn = self._dirName
-        else:
-            dn = os.path.basename(self._dirName)
-        
+        dn = self._dirName if full else os.path.basename(self._dirName)
         BrowserItem.__init__(self, parent, dn)
         
         self.type_ = BrowserItemDirectory
@@ -1068,11 +1069,7 @@ class BrowserDirectoryItem(BrowserItem):
         @param full flag indicating full pathname should be displayed (boolean)
         """
         self._dirName = os.path.abspath(dinfo)
-        
-        if full:
-            dn = self._dirName
-        else:
-            dn = os.path.basename(self._dirName)
+        dn = self._dirName if full else os.path.basename(self._dirName)
         self.itemData[0] = dn
     
     def dirName(self):
@@ -1100,9 +1097,11 @@ class BrowserDirectoryItem(BrowserItem):
         @param order sort order (Qt.SortOrder) (for special sorting)
         @return true, if this item is less than other (boolean)
         """
-        if issubclass(other.__class__, BrowserFileItem):
-            if Preferences.getUI("BrowsersListFoldersFirst"):
-                return order == Qt.AscendingOrder
+        if (
+            issubclass(other.__class__, BrowserFileItem) and
+            Preferences.getUI("BrowsersListFoldersFirst")
+        ):
+            return order == Qt.SortOrder.AscendingOrder
         
         return BrowserItem.lessThan(self, other, column, order)
 
@@ -1156,12 +1155,18 @@ class BrowserFileItem(BrowserItem):
         
         self._moduleName = ''
         
+        # TODO: add icon for eric graphics file
         pixName = ""
         if self.isPython3File():
             pixName = "filePython"
             self._populated = False
             self._lazyPopulation = True
             self._moduleName = os.path.basename(finfo)
+        elif self.isCythonFile():
+            pixName = "lexerCython"
+            self._populated = False
+            self._lazyPopulation = True
+            self._moduleName = os.path.basename(finfo)
         elif self.isRubyFile():
             pixName = "fileRuby"
             self._populated = False
@@ -1272,18 +1277,32 @@ class BrowserFileItem(BrowserItem):
         """
         Public method to check, if this file is a Python3 script.
         
-        @return flag indicating a Python file (boolean)
+        @return flag indicating a Python3 file
+        @rtype bool
         """
         return (
             self.fileext in Preferences.getPython("Python3Extensions") or
             (self.fileext == "" and self.sourceLanguage == "Python3")
         )
     
+    def isCythonFile(self):
+        """
+        Public method to check, if this file is a Cython file.
+        
+        @return flag indicating a Cython file
+        @rtype bool
+        """
+        return (
+            self.fileext in (".pyx", ".pxd", ".pxi") or
+            (self.fileext == "" and self.sourceLanguage == "Cython")
+        )
+    
     def isRubyFile(self):
         """
         Public method to check, if this file is a Ruby script.
         
-        @return flag indicating a Ruby file (boolean)
+        @return flag indicating a Ruby file
+        @rtype bool
         """
         return (
             self.fileext == '.rb' or
@@ -1294,7 +1313,8 @@ class BrowserFileItem(BrowserItem):
         """
         Public method to check, if this file is a Qt-Designer file.
         
-        @return flag indicating a Qt-Designer file (boolean)
+        @return flag indicating a Qt-Designer file
+        @rtype bool
         """
         return self.fileext == '.ui'
     
@@ -1302,7 +1322,8 @@ class BrowserFileItem(BrowserItem):
         """
         Public method to check, if this file is a Qt-Linguist file.
         
-        @return flag indicating a Qt-Linguist file (boolean)
+        @return flag indicating a Qt-Linguist file
+        @rtype bool
         """
         return self.fileext in ['.ts', '.qm']
     
@@ -1310,7 +1331,8 @@ class BrowserFileItem(BrowserItem):
         """
         Public method to check, if this file is a Qt-Resources file.
         
-        @return flag indicating a Qt-Resources file (boolean)
+        @return flag indicating a Qt-Resources file
+        @rtype bool
         """
         return self.fileext == '.qrc'
     
@@ -1318,23 +1340,26 @@ class BrowserFileItem(BrowserItem):
         """
         Public method to check, if this file is an eric project file.
         
-        @return flag indicating an eric project file (boolean)
+        @return flag indicating an eric project file
+        @rtype bool
         """
-        return self.fileext in ['.e4p']
+        return self.fileext in ('.epj', '.e4p')
     
     def isMultiProjectFile(self):
         """
         Public method to check, if this file is an eric multi project file.
         
-        @return flag indicating an eric project file (boolean)
+        @return flag indicating an eric project file
+        @rtype bool
         """
-        return self.fileext in ['.e4m', '.e5m']
+        return self.fileext in ('.emj', '.e4m', '.e5m')
     
     def isIdlFile(self):
         """
         Public method to check, if this file is a CORBA IDL file.
         
-        @return flag indicating a CORBA IDL file (boolean)
+        @return flag indicating a CORBA IDL file
+        @rtype bool
         """
         return self.fileext == '.idl'
     
@@ -1351,7 +1376,8 @@ class BrowserFileItem(BrowserItem):
         """
         Public method to check, if this file is a JavaScript file.
         
-        @return flag indicating a JavaScript file (boolean)
+        @return flag indicating a JavaScript file
+        @rtype bool
         """
         return self.fileext == '.js'
     
@@ -1359,7 +1385,8 @@ class BrowserFileItem(BrowserItem):
         """
         Public method to check, if this file is a pixmap file.
         
-        @return flag indicating a pixmap file (boolean)
+        @return flag indicating a pixmap file
+        @rtype bool
         """
         return self.fileext[1:] in QImageReader.supportedImageFormats()
     
@@ -1367,7 +1394,8 @@ class BrowserFileItem(BrowserItem):
         """
         Public method to check, if this file is a SVG file.
         
-        @return flag indicating a SVG file (boolean)
+        @return flag indicating a SVG file
+        @rtype bool
         """
         return self.fileext == '.svg'
     
@@ -1382,6 +1410,15 @@ class BrowserFileItem(BrowserItem):
             (self.fileext == "" and self.sourceLanguage == "D")
         )
     
+    def isEricGraphicsFile(self):
+        """
+        Public method to check, if this is an eric graphics file.
+        
+        @return flag indicating an eric graphics file
+        @rtype bool
+        """
+        return self.fileext in ('.egj', '.e5g')
+    
     def lessThan(self, other, column, order):
         """
         Public method to check, if the item is less than the other one.
@@ -1391,18 +1428,20 @@ class BrowserFileItem(BrowserItem):
         @param order sort order (Qt.SortOrder) (for special sorting)
         @return true, if this item is less than other (boolean)
         """
-        if not issubclass(other.__class__, BrowserFileItem):
-            if Preferences.getUI("BrowsersListFoldersFirst"):
-                return order == Qt.DescendingOrder
+        if (
+            not issubclass(other.__class__, BrowserFileItem) and
+            Preferences.getUI("BrowsersListFoldersFirst")
+        ):
+            return order == Qt.SortOrder.DescendingOrder
         
         if issubclass(other.__class__, BrowserFileItem):
             sinit = os.path.basename(self._filename).startswith('__init__.py')
             oinit = os.path.basename(other.fileName()).startswith(
                 '__init__.py')
             if sinit and not oinit:
-                return order == Qt.AscendingOrder
+                return order == Qt.SortOrder.AscendingOrder
             if not sinit and oinit:
-                return order == Qt.DescendingOrder
+                return order == Qt.SortOrder.DescendingOrder
         
         return BrowserItem.lessThan(self, other, column, order)
 
@@ -1430,7 +1469,7 @@ class BrowserClassItem(BrowserItem):
                 except AttributeError:
                     sname = sup
                 supers.append(sname)
-            name = name + "({0})".format(", ".join(supers))
+            name += "({0})".format(", ".join(supers))
         
         BrowserItem.__init__(self, parent, name)
         
@@ -1461,10 +1500,10 @@ class BrowserClassItem(BrowserItem):
             if self._classObject.annotation:
                 self.itemData[0] = "{0} {1}".format(
                     self.itemData[0], self._classObject.annotation)
-            ## if no defaults are wanted
-            ## ....format(name,
-            ##            ", ".join([e.split('=')[0].strip() \
-            ##                       for e in self._classObject.parameters]))
+            #- if no defaults are wanted
+            #- ....format(name,
+            #-            ", ".join([e.split('=')[0].strip()
+            #-                       for e in self._classObject.parameters]))
         elif self.ismodule:
             self.icon = UI.PixmapCache.getIcon("module")
         elif self.isenum:
@@ -1539,13 +1578,13 @@ class BrowserClassItem(BrowserItem):
             other.__class__,
             (BrowserCodingItem, BrowserClassAttributesItem)
         ):
-            return order == Qt.DescendingOrder
+            return order == Qt.SortOrder.DescendingOrder
         
         if (
             Preferences.getUI("BrowsersListContentsByOccurrence") and
             column == 0
         ):
-            if order == Qt.AscendingOrder:
+            if order == Qt.SortOrder.AscendingOrder:
                 return self.lineno() < other.lineno()
             else:
                 return self.lineno() > other.lineno()
@@ -1664,17 +1703,17 @@ class BrowserMethodItem(BrowserItem):
         """
         if issubclass(other.__class__, BrowserMethodItem):
             if self._name.startswith('__init__'):
-                return order == Qt.AscendingOrder
+                return order == Qt.SortOrder.AscendingOrder
             if other._name.startswith('__init__'):
-                return order == Qt.DescendingOrder
+                return order == Qt.SortOrder.DescendingOrder
         elif issubclass(other.__class__, BrowserClassAttributesItem):
-            return order == Qt.DescendingOrder
+            return order == Qt.SortOrder.DescendingOrder
         
         if (
             Preferences.getUI("BrowsersListContentsByOccurrence") and
             column == 0
         ):
-            if order == Qt.AscendingOrder:
+            if order == Qt.SortOrder.AscendingOrder:
                 return self.lineno() < other.lineno()
             else:
                 return self.lineno() > other.lineno()
@@ -1749,12 +1788,12 @@ class BrowserClassAttributesItem(BrowserItem):
         @return true, if this item is less than other (boolean)
         """
         if issubclass(other.__class__, BrowserCodingItem):
-            return order == Qt.DescendingOrder
+            return order == Qt.SortOrder.DescendingOrder
         elif issubclass(
             other.__class__,
             (BrowserClassItem, BrowserMethodItem)
         ):
-            return order == Qt.AscendingOrder
+            return order == Qt.SortOrder.AscendingOrder
         
         return BrowserItem.lessThan(self, other, column, order)
 
@@ -1838,7 +1877,7 @@ class BrowserClassAttributeItem(BrowserItem):
             Preferences.getUI("BrowsersListContentsByOccurrence") and
             column == 0
         ):
-            if order == Qt.AscendingOrder:
+            if order == Qt.SortOrder.AscendingOrder:
                 return self.lineno() < other.lineno()
             else:
                 return self.lineno() > other.lineno()
@@ -1905,7 +1944,7 @@ class BrowserCodingItem(BrowserItem):
             other.__class__,
             (BrowserClassItem, BrowserClassAttributesItem, BrowserImportItem)
         ):
-            return order == Qt.AscendingOrder
+            return order == Qt.SortOrder.AscendingOrder
         
         return BrowserItem.lessThan(self, other, column, order)
 
@@ -1939,7 +1978,7 @@ class BrowserImportsItem(BrowserItem):
             other.__class__,
             (BrowserClassItem, BrowserClassAttributesItem)
         ):
-            return order == Qt.AscendingOrder
+            return order == Qt.SortOrder.AscendingOrder
         
         return BrowserItem.lessThan(self, other, column, order)
 
@@ -2008,7 +2047,7 @@ class BrowserImportItem(BrowserItem):
             Preferences.getUI("BrowsersListContentsByOccurrence") and
             column == 0
         ):
-            if order == Qt.AscendingOrder:
+            if order == Qt.SortOrder.AscendingOrder:
                 return self.lineno() < other.lineno()
             else:
                 return self.lineno() > other.lineno()
