@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2005 - 2020 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2005 - 2021 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -10,7 +10,6 @@ Parse enough of a Python file to recognize class and method definitions and
 to find out the superclasses of a class as well as its attributes.
 """
 
-
 import sys
 import re
 from functools import reduce
@@ -65,7 +64,7 @@ _getnext = re.compile(
 |   (?P<Method>
         ^
         (?P<MethodIndent> [ \t]* )
-        (?: async [ \t]+ )? def [ \t]+
+        (?: async [ \t]+ )? (?: cdef | cpdef | def) [ \t]+
         (?P<MethodName> \w+ )
         (?: [ \t]* \[ (?: plain | html ) \] )?
         [ \t]* \(
@@ -78,6 +77,7 @@ _getnext = re.compile(
 |   (?P<Class>
         ^
         (?P<ClassIndent> [ \t]* )
+        (?: cdef [ \t]+ )?
         class [ \t]+
         (?P<ClassName> \w+ )
         [ \t]*
@@ -113,7 +113,7 @@ _getnext = re.compile(
     )
 
 |   (?P<Import>
-        ^ [ \t]* (?: import | from [ \t]+ \. [ \t]+ import ) [ \t]+
+        ^ [ \t]* (?: c? import | from [ \t]+ \. [ \t]+ c? import ) [ \t]+
         (?P<ImportList> (?: [^#;\\\n]* (?: \\\n )* )* )
     )
 
@@ -126,7 +126,7 @@ _getnext = re.compile(
             )*
         )
         [ \t]+
-        import [ \t]+
+        c? import [ \t]+
         (?P<ImportFromList>
             (?: \( \s* .*? \s* \) )
             |
@@ -215,7 +215,7 @@ class Attribute(ClbrBaseClasses.Attribute, VisibilityMixin):
         VisibilityMixin.__init__(self)
 
 
-class Publics(object):
+class Publics:
     """
     Class to represent the list of public identifiers.
     """
@@ -236,7 +236,7 @@ class Publics(object):
                             for e in idents.split(',')]
 
 
-class Imports(object):
+class Imports:
     """
     Class to represent the list of imported modules.
     """
@@ -289,7 +289,7 @@ class Imports(object):
         return self.imports
 
 
-class ImportedModule(object):
+class ImportedModule:
     """
     Class to represent an imported module.
     """
@@ -378,7 +378,7 @@ def readmodule_ex(module, path=None, inpackage=False, isPyFile=False):
 
     try:
         src = Utilities.readEncodedFile(file)[0]
-    except (UnicodeError, IOError):
+    except (UnicodeError, OSError):
         # can't do anything with this module
         _modules[module] = {}
         return {}
@@ -437,7 +437,7 @@ def scan(src, file, module):
     conditionalsstack = []  # stack of indents of conditional defines
     deltastack = []
     deltaindent = 0
-    deltaindentcalculated = 0
+    deltaindentcalculated = False
     
     lineno, last_lineno_pos = 1, 0
     i = 0
@@ -463,7 +463,7 @@ def scan(src, file, module):
             meth_ret = m.group("MethodReturnAnnotation")
             meth_ret = meth_ret.replace('\\\n', '')
             meth_ret = _commentsub('', meth_ret)
-            lineno = lineno + src.count('\n', last_lineno_pos, start)
+            lineno += src.count('\n', last_lineno_pos, start)
             last_lineno_pos = start
             if modifierType and modifierIndent == thisindent:
                 if modifierType == "@staticmethod":
@@ -480,7 +480,7 @@ def scan(src, file, module):
                     if not deltaindentcalculated:
                         deltastack.append(thisindent - conditionalsstack[-1])
                         deltaindent = reduce(lambda x, y: x + y, deltastack)
-                        deltaindentcalculated = 1
+                        deltaindentcalculated = True
                     thisindent -= deltaindent
                 else:
                     while (
@@ -490,7 +490,7 @@ def scan(src, file, module):
                         del conditionalsstack[-1]
                         if deltastack:
                             del deltastack[-1]
-                    deltaindentcalculated = 0
+                    deltaindentcalculated = False
             # close all classes indented at least as much
             while classstack and classstack[-1][1] >= thisindent:
                 del classstack[-1]
@@ -535,7 +535,7 @@ def scan(src, file, module):
             # close all classes indented at least as much
             while classstack and classstack[-1][1] >= thisindent:
                 del classstack[-1]
-            lineno = lineno + src.count('\n', last_lineno_pos, start)
+            lineno += src.count('\n', last_lineno_pos, start)
             last_lineno_pos = start
             class_name = m.group("ClassName")
             inherit = m.group("ClassSupers")
@@ -561,10 +561,26 @@ def scan(src, file, module):
                             c = c[-1]
                             if m in _modules:
                                 d = _modules[m]
-                                if c in d:
-                                    n = d[c]
+                                n = d.get(c, n)
                     names.append(n)
                 inherit = names
+            # modify indentation level for conditional defines
+            if conditionalsstack:
+                if thisindent > conditionalsstack[-1]:
+                    if not deltaindentcalculated:
+                        deltastack.append(thisindent - conditionalsstack[-1])
+                        deltaindent = reduce(lambda x, y: x + y, deltastack)
+                        deltaindentcalculated = True
+                    thisindent -= deltaindent
+                else:
+                    while (
+                        conditionalsstack and
+                        conditionalsstack[-1] >= thisindent
+                    ):
+                        del conditionalsstack[-1]
+                        if deltastack:
+                            del deltastack[-1]
+                    deltaindentcalculated = False
             # remember this class
             cur_class = Class(module, class_name, inherit,
                               file, lineno)
@@ -583,7 +599,7 @@ def scan(src, file, module):
             classstack.append((cur_class, thisindent))
 
         elif m.start("Attribute") >= 0:
-            lineno = lineno + src.count('\n', last_lineno_pos, start)
+            lineno += src.count('\n', last_lineno_pos, start)
             last_lineno_pos = start
             index = -1
             while index >= -len(classstack):
@@ -600,14 +616,14 @@ def scan(src, file, module):
 
         elif m.start("Main") >= 0:
             # 'main' part of the script, reset class stack
-            lineno = lineno + src.count('\n', last_lineno_pos, start)
+            lineno += src.count('\n', last_lineno_pos, start)
             last_lineno_pos = start
             classstack = []
 
         elif m.start("Variable") >= 0:
             thisindent = _indent(m.group("VariableIndent"))
             variable_name = m.group("VariableName")
-            lineno = lineno + src.count('\n', last_lineno_pos, start)
+            lineno += src.count('\n', last_lineno_pos, start)
             last_lineno_pos = start
             if thisindent == 0 or not classstack:
                 # global variable, reset class stack first
@@ -631,17 +647,17 @@ def scan(src, file, module):
 
         elif m.start("Publics") >= 0:
             idents = m.group("Identifiers")
-            lineno = lineno + src.count('\n', last_lineno_pos, start)
+            lineno += src.count('\n', last_lineno_pos, start)
             last_lineno_pos = start
             pubs = Publics(module, file, lineno, idents)
             dictionary['__all__'] = pubs
         
         elif m.start("Import") >= 0:
-            ## import module
+            #- import module
             names = [n.strip() for n in
                      "".join(m.group("ImportList").splitlines())
                      .replace("\\", "").split(',')]
-            lineno = lineno + src.count('\n', last_lineno_pos, start)
+            lineno += src.count('\n', last_lineno_pos, start)
             last_lineno_pos = start
             if "@@Import@@" not in dictionary:
                 dictionary["@@Import@@"] = Imports(module, file)
@@ -649,7 +665,7 @@ def scan(src, file, module):
                 dictionary["@@Import@@"].addImport(name, [], lineno)
         
         elif m.start("ImportFrom") >= 0:
-            ## from module import stuff
+            #- from module import stuff
             mod = m.group("ImportFromPath")
             namesLines = (m.group("ImportFromList")
                           .replace("(", "").replace(")", "")
@@ -660,7 +676,7 @@ def scan(src, file, module):
             names = [n.strip() for n in
                      "".join(namesLines)
                      .split(',')]
-            lineno = lineno + src.count('\n', last_lineno_pos, start)
+            lineno += src.count('\n', last_lineno_pos, start)
             last_lineno_pos = start
             if "@@Import@@" not in dictionary:
                 dictionary["@@Import@@"] = Imports(module, file)
@@ -674,12 +690,12 @@ def scan(src, file, module):
                 if deltastack:
                     del deltastack[-1]
             conditionalsstack.append(thisindent)
-            deltaindentcalculated = 0
+            deltaindentcalculated = False
         
         elif m.start("CodingLine") >= 0:
             # a coding statement
             coding = m.group("Coding")
-            lineno = lineno + src.count('\n', last_lineno_pos, start)
+            lineno += src.count('\n', last_lineno_pos, start)
             last_lineno_pos = start
             if "@@Coding@@" not in dictionary:
                 dictionary["@@Coding@@"] = ClbrBaseClasses.Coding(
@@ -688,7 +704,7 @@ def scan(src, file, module):
     if '__all__' in dictionary:
         # set visibility of all top level elements
         pubs = dictionary['__all__']
-        for key in dictionary.keys():
+        for key in dictionary:
             if key == '__all__' or key.startswith("@@"):
                 continue
             if key in pubs.identifiers:
