@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2003 - 2020 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2003 - 2021 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -14,11 +14,11 @@ Parse a Python module file.
 </ul>
 """
 
-
 import sys
 import os
 import importlib.machinery
 import re
+import contextlib
 
 import Utilities
 from functools import reduce
@@ -38,6 +38,7 @@ TYPE_MAPPING = {
     "Python": PY_SOURCE,
     "Python3": PY_SOURCE,
     "MicroPython": PY_SOURCE,
+    "Cython": PY_SOURCE,
     "Ruby": RB_SOURCE,
 }
 
@@ -127,15 +128,15 @@ _py_getnext = re.compile(
     )
 
 |   (?P<Method>
-        (^ [ \t]* @ (?: PyQt[45] \. )? (?: QtCore \. )?
-            (?: pyqtSignature | pyqtSlot )
+        (^ [ \t]* @ (?: PyQt[456] \. | PySide[26] \. )? (?: QtCore \. )?
+            (?: pyqtSignature | pyqtSlot | Slot )
             [ \t]* \(
                 (?P<MethodPyQtSignature> [^)]* )
             \) \s*
         )?
         ^
         (?P<MethodIndent> [ \t]* )
-        (?: async [ \t]+ )? def [ \t]+
+        (?: async [ \t]+ )? (?: cdef | cpdef | def) [ \t]+
         (?P<MethodName> \w+ )
         (?: [ \t]* \[ (?: plain | html ) \] )?
         [ \t]* \(
@@ -148,6 +149,7 @@ _py_getnext = re.compile(
 |   (?P<Class>
         ^
         (?P<ClassIndent> [ \t]* )
+        (?: cdef [ \t]+ )?
         class [ \t]+
         (?P<ClassName> \w+ )
         [ \t]*
@@ -176,7 +178,7 @@ _py_getnext = re.compile(
     )
 
 |   (?P<Import>
-        ^ [ \t]* (?: import | from [ \t]+ \. [ \t]+ import ) [ \t]+
+        ^ [ \t]* (?: c? import | from [ \t]+ \. [ \t]+ c? import ) [ \t]+
         (?P<ImportList> (?: [^#;\\\n]* (?: \\\n )* )* )
     )
 
@@ -189,7 +191,7 @@ _py_getnext = re.compile(
             )*
         )
         [ \t]+
-        import [ \t]+
+        c? import [ \t]+
         (?P<ImportFromList>
             (?: \( \s* .*? \s* \) )
             |
@@ -348,7 +350,7 @@ _commentsub = re.compile(r"""#[^\n]*\n|#[^\n]*$""").sub
 _modules = {}                           # cache of modules we've seen
 
 
-class VisibilityBase(object):
+class VisibilityBase:
     """
     Class implementing the visibility aspect of all objects.
     """
@@ -395,7 +397,7 @@ class VisibilityBase(object):
         self.visibility = 2
 
 
-class Module(object):
+class Module:
     """
     Class to represent a Python module.
     """
@@ -592,7 +594,7 @@ class Module(object):
                     )
                 else:
                     meth_pyqtSig = None
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 if modifierType and modifierIndent == thisindent:
                     if modifierType == "@staticmethod":
@@ -707,7 +709,7 @@ class Module(object):
             elif m.start("Class") >= 0:
                 # we found a class definition
                 thisindent = _indent(m.group("ClassIndent"))
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 # close all classes indented at least as much
                 while classstack and classstack[-1][1] >= thisindent:
@@ -738,6 +740,27 @@ class Module(object):
                                         n = m.name
                             names.append(n)
                     inherit = names
+                # modify indentation level for conditional defines
+                if conditionalsstack:
+                    if thisindent > conditionalsstack[-1]:
+                        if not deltaindentcalculated:
+                            deltastack.append(
+                                thisindent - conditionalsstack[-1]
+                            )
+                            deltaindent = reduce(
+                                lambda x, y: x + y, deltastack
+                            )
+                            deltaindentcalculated = True
+                        thisindent -= deltaindent
+                    else:
+                        while (
+                            conditionalsstack and
+                            conditionalsstack[-1] >= thisindent
+                        ):
+                            del conditionalsstack[-1]
+                            if deltastack:
+                                del deltastack[-1]
+                        deltaindentcalculated = False
                 # remember this class
                 cur_class = Class(self.name, class_name, inherit,
                                   self.file, lineno)
@@ -750,7 +773,7 @@ class Module(object):
                 classstack.append((cur_class, thisindent))
             
             elif m.start("Attribute") >= 0:
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 index = -1
                 while index >= -len(classstack):
@@ -766,7 +789,7 @@ class Module(object):
             
             elif m.start("Main") >= 0:
                 # 'main' part of the script, reset class stack
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 classstack = []
             
@@ -774,7 +797,7 @@ class Module(object):
                 thisindent = _indent(m.group("VariableIndent"))
                 variable_name = m.group("VariableName")
                 isSignal = m.group("VariableSignal") != ""
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 if thisindent == 0:
                     # global variable
@@ -802,7 +825,7 @@ class Module(object):
                             break
 
             elif m.start("Import") >= 0:
-                ## import module
+                #- import module
                 names = [n.strip() for n in
                          "".join(m.group("ImportList").splitlines())
                          .replace("\\", "").split(',')]
@@ -811,7 +834,7 @@ class Module(object):
                      if name not in self.imports])
             
             elif m.start("ImportFrom") >= 0:
-                ## from module import stuff
+                #- from module import stuff
                 mod = m.group("ImportFromPath")
                 namesLines = (m.group("ImportFromList")
                               .replace("(", "").replace(")", "")
@@ -841,9 +864,8 @@ class Module(object):
                 conditionalsstack.append(thisindent)
                 deltaindentcalculated = 0
             
-            elif m.start("Comment") >= 0:
-                if modulelevel:
-                    continue
+            elif m.start("Comment") >= 0 and modulelevel:
+                continue
             
             modulelevel = False
     
@@ -852,7 +874,8 @@ class Module(object):
         Private method to scan the source text of a Python module and retrieve
         the relevant information.
         
-        @param src the source text to be scanned (string)
+        @param src the source text to be scanned
+        @type str
         """
         lineno, last_lineno_pos = 1, 0
         classstack = []  # stack of (class, indent) pairs
@@ -878,7 +901,7 @@ class Module(object):
                 )
                 meth_sig = m.group("MethodSignature")
                 meth_sig = meth_sig and meth_sig.replace('\\\n', '') or ''
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 if meth_name.startswith('self.'):
                     meth_name = meth_name[5:]
@@ -905,10 +928,7 @@ class Module(object):
                         if cur_class is None:
                             continue
                         
-                        if (
-                            isinstance(cur_class, Class) or
-                            isinstance(cur_class, RbModule)
-                        ):
+                        if isinstance(cur_class, (Class, RbModule)):
                             # it's a class/module method
                             f = Function(None, meth_name,
                                          None, lineno, meth_sig)
@@ -949,20 +969,11 @@ class Module(object):
                 if cur_obj:
                     cur_obj.addDescription(contents)
             
-            elif m.start("String") >= 0:
-                pass
-            
-            elif m.start("Comment") >= 0:
-                pass
-            
-            elif m.start("ClassIgnored") >= 0:
-                pass
-            
             elif m.start("Class") >= 0:
                 # we found a class definition
                 thisindent = indent
                 indent += 1
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 # close all classes/modules indented at least as much
                 while classstack and classstack[-1][1] >= thisindent:
@@ -1013,7 +1024,7 @@ class Module(object):
                 # we found a module definition
                 thisindent = indent
                 indent += 1
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 # close all classes/modules indented at least as much
                 while classstack and classstack[-1][1] >= thisindent:
@@ -1065,7 +1076,7 @@ class Module(object):
                         if (
                             classstack[index][0] is not None and
                             not isinstance(classstack[index][0], Function) and
-                            not classstack[index][1] >= indent
+                            classstack[index][1] < indent
                         ):
                             parent = classstack[index][0]
                             actype = (
@@ -1090,14 +1101,14 @@ class Module(object):
                             index -= 1
 
             elif m.start("Attribute") >= 0:
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 index = -1
                 while index >= -len(classstack):
                     if (
                         classstack[index][0] is not None and
                         not isinstance(classstack[index][0], Function) and
-                        not classstack[index][1] >= indent
+                        classstack[index][1] < indent
                     ):
                         attrName = m.group("AttributeName")
                         attr = Attribute(
@@ -1120,14 +1131,14 @@ class Module(object):
                     lastGlobalEntry = None
             
             elif m.start("Attr") >= 0:
-                lineno = lineno + src.count('\n', last_lineno_pos, start)
+                lineno += src.count('\n', last_lineno_pos, start)
                 last_lineno_pos = start
                 index = -1
                 while index >= -len(classstack):
                     if (
                         classstack[index][0] is not None and
                         not isinstance(classstack[index][0], Function) and
-                        not classstack[index][1] >= indent
+                        classstack[index][1] < indent
                     ):
                         parent = classstack[index][0]
                         if m.group("AttrType") is None:
@@ -1161,10 +1172,7 @@ class Module(object):
                                 )
                                 if access == "_accessor":
                                     attr.setPublic()
-                                elif (
-                                    access == "_reader" or
-                                    access == "_writer"
-                                ):
+                                elif access in ("_reader", "_writer"):
                                     if attr.isPrivate():
                                         attr.setProtected()
                                     elif attr.isProtected():
@@ -1189,7 +1197,12 @@ class Module(object):
                     else:
                         indent = 0
             
-            elif m.start("BeginEnd") >= 0:
+            elif (
+                m.start("String") >= 0 or
+                m.start("Comment") >= 0 or
+                m.start("ClassIgnored") >= 0 or
+                m.start("BeginEnd") >= 0
+            ):
                 pass
     
     def createHierarchy(self):
@@ -1480,7 +1493,7 @@ class Attribute(VisibilityBase):
         @param name name of the function (string)
         @param file name of file containing this function (string)
         @param lineno linenumber of the first attribute assignment (integer)
-        @keyparam isSignal flag indicating a signal definition (boolean)
+        @param isSignal flag indicating a signal definition (boolean)
         """
         self.module = module
         self.name = name
@@ -1527,14 +1540,13 @@ def readModule(module, path=None, inpackage=False, basename="",
     """
     global _modules
     
-    if extensions is None:
-        _extensions = ['.py', '.pyw', '.ptl', '.rb']
-    else:
-        _extensions = extensions[:]
-    try:
+    _extensions = (
+        ['.py', '.pyw', '.ptl', '.rb']
+        if extensions is None else
+        extensions[:]
+    )
+    with contextlib.suppress(ValueError):
         _extensions.remove('.py')
-    except ValueError:
-        pass
     
     modname = module
     
@@ -1595,11 +1607,9 @@ def readModule(module, path=None, inpackage=False, basename="",
         return _modules[modname]
     
     mod = Module(modname, file, moduleType)
-    try:
+    with contextlib.suppress(UnicodeError, OSError):
         src = Utilities.readEncodedFile(file)[0]
         mod.scan(src)
-    except (UnicodeError, IOError):
-        pass
     if caching:
         _modules[modname] = mod
     return mod
@@ -1639,15 +1649,24 @@ def find_module(name, path, extensions):
                     pathname = os.path.join(p, name)
                     if ext == '.ptl':
                         # Quixote page template
-                        return (open(pathname), pathname,
-                                ('.ptl', 'r', PTL_SOURCE))
+                        return (
+                            open(pathname), pathname,
+                            # __IGNORE_WARNING_Y115__
+                            ('.ptl', 'r', PTL_SOURCE)
+                        )
                     elif ext == '.rb':
                         # Ruby source file
-                        return (open(pathname), pathname,
-                                ('.rb', 'r', RB_SOURCE))
+                        return (
+                            open(pathname), pathname,
+                            # __IGNORE_WARNING_Y115__
+                            ('.rb', 'r', RB_SOURCE)
+                        )
                     else:
-                        return (open(pathname), pathname,
-                                (ext, 'r', PY_SOURCE))
+                        return (
+                            open(pathname), pathname,
+                            # __IGNORE_WARNING_Y115__
+                            (ext, 'r', PY_SOURCE)
+                        )
             raise ImportError
     
     # standard Python module file
@@ -1660,6 +1679,7 @@ def find_module(name, path, extensions):
     if isinstance(spec.loader, importlib.machinery.SourceFileLoader):
         ext = os.path.splitext(spec.origin)[-1]
         return (open(spec.origin), spec.origin, (ext, 'r', PY_SOURCE))
+        # __IGNORE_WARNING_Y115__
     
     raise ImportError
 
