@@ -1,13 +1,12 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2003 - 2020 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2003 - 2021 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
-Package implementing various functions/classes needed everywhere within eric6.
+Package implementing various functions/classes needed everywhere within eric.
 """
 
-
 import os
 import sys
 import codecs
@@ -17,6 +16,8 @@ import glob
 import getpass
 import ctypes
 import subprocess           # secok
+import shlex
+import contextlib
 
 
 def __showwarning(message, category, filename, lineno, file=None, line=""):
@@ -43,8 +44,8 @@ warnings.showwarning = __showwarning
 from codecs import BOM_UTF8, BOM_UTF16, BOM_UTF32
 
 from PyQt5.QtCore import (
-    QRegExp, QDir, QProcess, Qt, QByteArray, qVersion, PYQT_VERSION_STR,
-    QCoreApplication, QCryptographicHash
+    qVersion, PYQT_VERSION_STR, QDir, QProcess, QByteArray, QCoreApplication,
+    QCryptographicHash
 )
 from PyQt5.Qsci import QSCINTILLA_VERSION_STR, QsciScintilla
 
@@ -79,21 +80,40 @@ coding_regexps = [
 
 supportedCodecs = [
     'utf-8',
-    'iso8859-1', 'iso8859-15', 'iso8859-2', 'iso8859-3',
-    'iso8859-4', 'iso8859-5', 'iso8859-6', 'iso8859-7',
-    'iso8859-8', 'iso8859-9', 'iso8859-10', 'iso8859-11',
-    'iso8859-13', 'iso8859-14', 'iso8859-16', 'latin-1',
-    'koi8-r', 'koi8-u',
-    'utf-16', 'utf-32',
-    'cp037', 'cp424', 'cp437', 'cp500', 'cp737', 'cp775',
-    'cp850', 'cp852', 'cp855', 'cp856', 'cp857', 'cp860',
-    'cp861', 'cp862', 'cp863', 'cp864', 'cp865', 'cp866',
-    'cp869', 'cp874', 'cp875', 'cp932', 'cp949', 'cp950',
-    'cp1006', 'cp1026', 'cp1140', 'cp1250', 'cp1251',
-    'cp1252', 'cp1253', 'cp1254', 'cp1255', 'cp1256',
-    'cp1257', 'cp1258',
-    'gb2312', 'gb18030',
-    'ascii'
+    
+    'iso-8859-1', 'iso-8859-2', 'iso-8859-3',
+    'iso-8859-4', 'iso-8859-5', 'iso-8859-6', 'iso-8859-7',
+    'iso-8859-8', 'iso-8859-9', 'iso-8859-10', 'iso-8859-11',
+    'iso-8859-13', 'iso-8859-14', 'iso-8859-15', 'iso-8859-16',
+    'latin-1',
+    
+    'koi8-r', 'koi8-t', 'koi8-u',
+    
+    'utf-7',
+    'utf-16', 'utf-16-be', 'utf-16-le',
+    'utf-32', 'utf-32-be', 'utf-32-le',
+    
+    'cp037', 'cp273', 'cp424', 'cp437', 'cp500', 'cp720',
+    'cp737', 'cp775', 'cp850', 'cp852', 'cp855', 'cp856',
+    'cp857', 'cp858', 'cp860', 'cp861', 'cp862', 'cp863',
+    'cp864', 'cp865', 'cp866', 'cp869', 'cp874', 'cp875',
+    'cp932', 'cp949', 'cp950', 'cp1006', 'cp1026', 'cp1125',
+    'cp1140',
+    
+    'windows-1250', 'windows-1251', 'windows-1252', 'windows-1253',
+    'windows-1254', 'windows-1255', 'windows-1256', 'windows-1257',
+    'windows-1258',
+    
+    'gb2312', 'hz', 'gb18030', 'gbk',
+    
+    'iso-2022-jp', 'iso-2022-jp-1', 'iso-2022-jp-2', 'iso-2022-jp-2004',
+    'iso-2022-jp-3', 'iso-2022-jp-ext', 'iso-2022-kr',
+    
+    'mac-cyrillic', 'mac-greek', 'mac-iceland', 'mac-latin2',
+    'mac-roman', 'mac-turkish',
+    
+    'ascii',
+    'big5-tw', 'big5-hkscs',
 ]
 
 
@@ -172,9 +192,8 @@ def readEncodedFile(filename):
     @param filename name of the file to read (string)
     @return tuple of decoded text and encoding (string, string)
     """
-    f = open(filename, "rb")
-    text = f.read()
-    f.close()
+    with open(filename, "rb") as f:
+        text = f.read()
     return decode(text)
 
 
@@ -187,11 +206,11 @@ def readEncodedFileWithHash(filename):
     @return tuple of decoded text, encoding and hash value (string, string,
         string)
     """
-    f = open(filename, "rb")
-    text = f.read()
-    f.close()
+    with open(filename, "rb") as f:
+        text = f.read()
     hashStr = str(QCryptographicHash.hash(
-        QByteArray(text), QCryptographicHash.Md5).toHex(), encoding="ASCII")
+        QByteArray(text), QCryptographicHash.Algorithm.Md5).toHex(),
+        encoding="ASCII")
     return decode(text) + (hashStr, )
 
 
@@ -202,7 +221,7 @@ def decode(text):
     @param text byte text to decode (bytes)
     @return tuple of decoded text and encoding (string, string)
     """
-    try:
+    with contextlib.suppress(UnicodeError, LookupError):
         if text.startswith(BOM_UTF8):
             # UTF-8 with BOM
             return str(text[len(BOM_UTF8):], 'utf-8'), 'utf-8-bom'
@@ -215,21 +234,17 @@ def decode(text):
         coding = get_codingBytes(text)
         if coding:
             return str(text, coding), coding
-    except (UnicodeError, LookupError):
-        pass
     
     # Assume UTF-8
-    try:
+    with contextlib.suppress(UnicodeError, LookupError):
         return str(text, 'utf-8'), 'utf-8-guessed'
-    except (UnicodeError, LookupError):
-        pass
     
     guess = None
     if Preferences.getEditor("AdvancedEncodingDetection"):
         # Try the universal character encoding detector
         try:
-            import ThirdParty.CharDet.chardet
-            guess = ThirdParty.CharDet.chardet.detect(text)
+            import chardet
+            guess = chardet.detect(text)
             if (
                 guess and
                 guess['confidence'] > 0.95 and
@@ -243,20 +258,19 @@ def decode(text):
             pass
     
     # Try default encoding
-    try:
+    with contextlib.suppress(UnicodeError, LookupError):
         codec = Preferences.getEditor("DefaultEncoding")
         return str(text, codec), '{0}-default'.format(codec)
-    except (UnicodeError, LookupError):
-        pass
     
-    if Preferences.getEditor("AdvancedEncodingDetection"):
-        # Use the guessed one even if confifence level is low
-        if guess and guess['encoding'] is not None:
-            try:
-                codec = guess['encoding'].lower()
-                return str(text, codec), '{0}-guessed'.format(codec)
-            except (UnicodeError, LookupError):
-                pass
+    if (
+        Preferences.getEditor("AdvancedEncodingDetection") and
+        guess and
+        guess['encoding'] is not None
+    ):
+        # Use the guessed one even if confidence level is low
+        with contextlib.suppress(UnicodeError, LookupError):
+            codec = guess['encoding'].lower()
+            return str(text, codec), '{0}-guessed'.format(codec)
     
     # Assume UTF-8 loosing information
     return str(text, "utf-8", "ignore"), 'utf-8-ignore'
@@ -267,23 +281,20 @@ def readEncodedFileWithEncoding(filename, encoding):
     Function to read a file and decode its contents into proper text.
     
     @param filename name of the file to read (string)
-    @keyparam encoding encoding to be used to read the file (string)
+    @param encoding encoding to be used to read the file (string)
     @return tuple of decoded text and encoding (string, string)
     """
-    f = open(filename, "rb")
-    text = f.read()
-    f.close()
+    with open(filename, "rb") as f:
+        text = f.read()
     if encoding:
-        try:
+        with contextlib.suppress(UnicodeError, LookupError):
             return str(text, encoding), '{0}-selected'.format(encoding)
-        except (UnicodeError, LookupError):
-            pass
+        
         # Try default encoding
-        try:
+        with contextlib.suppress(UnicodeError, LookupError):
             codec = Preferences.getEditor("DefaultEncoding")
             return str(text, codec), '{0}-default'.format(codec)
-        except (UnicodeError, LookupError):
-            pass
+        
         # Assume UTF-8 loosing information
         return str(text, "utf-8", "ignore"), 'utf-8-ignore'
     else:
@@ -308,9 +319,8 @@ def writeEncodedFile(filename, text, origEncoding, forcedEncoding=""):
     """
     etext, encoding = encode(text, origEncoding, forcedEncoding=forcedEncoding)
     
-    f = open(filename, "wb")
-    f.write(etext)
-    f.close()
+    with open(filename, "wb") as f:
+        f.write(etext)
     
     return encoding
 
@@ -344,12 +354,10 @@ def encode(text, origEncoding, forcedEncoding=""):
                 raise CodingError(coding)
         else:
             if forcedEncoding:
-                try:
+                with contextlib.suppress(UnicodeError, LookupError):
                     etext, encoding = (
                         text.encode(forcedEncoding), forcedEncoding)
-                except (UnicodeError, LookupError):
-                    # Error: Forced encoding is incorrect, ignore it
-                    pass
+                    # if forced encoding is incorrect, ignore it
             
             if encoding is None:
                 # Try the original encoding
@@ -362,25 +370,19 @@ def encode(text, origEncoding, forcedEncoding=""):
                         .replace("-guessed", "")
                         .replace("-ignore", "")
                     )
-                    try:
+                    with contextlib.suppress(UnicodeError, LookupError):
                         etext, encoding = text.encode(coding), coding
-                    except (UnicodeError, LookupError):
-                        pass
                 
                 if encoding is None:
                     # Try configured default
-                    try:
+                    with contextlib.suppress(UnicodeError, LookupError):
                         codec = Preferences.getEditor("DefaultEncoding")
                         etext, encoding = text.encode(codec), codec
-                    except (UnicodeError, LookupError):
-                        pass
                     
                     if encoding is None:
                         # Try saving as ASCII
-                        try:
+                        with contextlib.suppress(UnicodeError):
                             etext, encoding = text.encode('ascii'), 'ascii'
-                        except UnicodeError:
-                            pass
                         
                         if encoding is None:
                             # Save as UTF-8 without BOM
@@ -418,7 +420,7 @@ def decodeBytes(buffer):
     @return decoded text (string)
     """
     # try UTF with BOM
-    try:
+    with contextlib.suppress(UnicodeError, LookupError):
         if buffer.startswith(BOM_UTF8):
             # UTF-8 with BOM
             return str(buffer[len(BOM_UTF8):], encoding='utf-8')
@@ -428,19 +430,15 @@ def decodeBytes(buffer):
         elif buffer.startswith(BOM_UTF32):
             # UTF-32 with BOM
             return str(buffer[len(BOM_UTF32):], encoding='utf-32')
-    except (UnicodeError, LookupError):
-        pass
     
     # try UTF-8
-    try:
+    with contextlib.suppress(UnicodeError):
         return str(buffer, encoding="utf-8")
-    except UnicodeError:
-        pass
     
     # try codec detection
     try:
-        import ThirdParty.CharDet.chardet
-        guess = ThirdParty.CharDet.chardet.detect(buffer)
+        import chardet
+        guess = chardet.detect(buffer)
         if guess and guess['encoding'] is not None:
             codec = guess['encoding'].lower()
             return str(buffer, encoding=codec)
@@ -589,9 +587,9 @@ def linesep():
     @return line separator used by the editor (string)
     """
     eolMode = Preferences.getEditor("EOLMode")
-    if eolMode == QsciScintilla.EolUnix:
+    if eolMode == QsciScintilla.EolMode.EolUnix:
         return "\n"
-    elif eolMode == QsciScintilla.EolMac:
+    elif eolMode == QsciScintilla.EolMode.EolMac:
         return "\r"
     else:
         return "\r\n"
@@ -613,10 +611,7 @@ def extractFlags(text):
     @return dictionary of string, boolean, complex, float and int
     """
     flags = {}
-    if isinstance(text, str):
-        lines = text.rstrip().splitlines()
-    else:
-        lines = text
+    lines = text.rstrip().splitlines() if isinstance(text, str) else text
     for line in reversed(lines):
         try:
             index = line.index("eflag:")
@@ -639,11 +634,9 @@ def extractFlags(text):
                 # interpret as int first
                 value = int(value)
             except ValueError:
-                try:
+                with contextlib.suppress(ValueError):
                     # interpret as float next
                     value = float(value)
-                except ValueError:
-                    pass
             
             flags[key] = value
         else:
@@ -666,7 +659,7 @@ def extractFlagsFromFile(filename):
     """
     try:
         source, encoding = readEncodedFile(filename)
-    except (UnicodeError, IOError):
+    except (UnicodeError, OSError):
         return {}
     
     return extractFlags(source)
@@ -678,9 +671,9 @@ def extractLineFlags(line, startComment="#", endComment="", flagsLine=False):
     comment.
     
     @param line line to extract flags from (string)
-    @keyparam startComment string identifying the start of the comment (string)
-    @keyparam endComment string identifying the end of a comment (string)
-    @keyparam flagsLine flag indicating to check for a flags only line (bool)
+    @param startComment string identifying the start of the comment (string)
+    @param endComment string identifying the end of a comment (string)
+    @param flagsLine flag indicating to check for a flags only line (bool)
     @return list containing the extracted flags (list of strings)
     """
     flags = []
@@ -729,7 +722,7 @@ def toNativeSeparators(path):
     @return path with converted separator characters (string)
     """
     return QDir.toNativeSeparators(path)
-    
+
 
 def fromNativeSeparators(path):
     """
@@ -739,7 +732,7 @@ def fromNativeSeparators(path):
     @return path with converted separator characters (string)
     """
     return QDir.fromNativeSeparators(path)
-    
+
 
 def normcasepath(path):
     """
@@ -750,17 +743,7 @@ def normcasepath(path):
     @return case normalized path (string)
     """
     return os.path.normcase(os.path.normpath(path))
-    
 
-def normabspath(path):
-    """
-    Function returning a normalized, absolute path.
-    
-    @param path file path (string)
-    @return absolute, normalized path (string)
-    """
-    return os.path.abspath(path)
-    
 
 def normcaseabspath(path):
     """
@@ -771,18 +754,18 @@ def normcaseabspath(path):
     @return absolute, normalized path (string)
     """
     return os.path.normcase(os.path.abspath(path))
-    
+
 
 def normjoinpath(a, *p):
     """
     Function returning a normalized path of the joined parts passed into it.
     
     @param a first path to be joined (string)
-    @param p variable number of path parts to be joind (string)
+    @param p variable number of path parts to be joined (string)
     @return normalized path (string)
     """
     return os.path.normpath(os.path.join(a, *p))
-    
+
 
 def normabsjoinpath(a, *p):
     """
@@ -794,30 +777,6 @@ def normabsjoinpath(a, *p):
     @return absolute, normalized path (string)
     """
     return os.path.abspath(os.path.join(a, *p))
-    
-
-def relpath(path, start=os.path.curdir):
-    """
-    Return a relative version of a path.
-    
-    @param path path to make relative (string)
-    @param start path to make relative from (string)
-    @return relative path (string)
-    @exception ValueError raised to indicate an invalid path
-    """
-    if not path:
-        raise ValueError("no path specified")
-
-    start_list = os.path.abspath(start).split(os.path.sep)
-    path_list = os.path.abspath(path).split(os.path.sep)
-
-    # Work out how much of the filepath is shared by start and path.
-    i = len(os.path.commonprefix([start_list, path_list]))
-
-    rel_list = [os.path.pardir] * (len(start_list) - i) + path_list[i:]
-    if not rel_list:
-        return os.path.curdir
-    return os.path.join(*rel_list)
 
 
 def isinpath(file):
@@ -841,53 +800,29 @@ def isinpath(file):
         return False
     
     dirs = path.split(os.pathsep)
-    for directory in dirs:
-        if os.access(os.path.join(directory, file), os.X_OK):
-            return True
-    
-    return False
+    return any(os.access(os.path.join(directory, file), os.X_OK)
+               for directory in dirs)
 
 
 def startswithPath(path, start):
     """
     Function to check, if a path starts with a given start path.
     
-    @param path path to be checked (string)
-    @param start start path (string)
+    @param path path to be checked
+    @type str
+    @param start start path
+    @type str
     @return flag indicating that the path starts with the given start
-        path (boolean)
+        path
+    @rtype bool
     """
-    if start:
-        if path == start:
-            return True
-        elif normcasepath(toNativeSeparators(path)).startswith(
-                normcasepath(toNativeSeparators(start + "/"))):
-            return True
-        else:
-            return False
-    else:
-        return False
-
-
-def relativePath(path, start):
-    """
-    Function to convert a file path to a path relative to a start path.
-    
-    @param path file or directory name to convert (string)
-    @param start start path (string)
-    @return relative path or unchanged path, if path does not start with
-        the start path (string)
-    """
-    if startswithPath(path, start):
-        if path == start:
-            return ""
-        else:
-            if start.endswith(("/", "\\")):
-                return path[len(start):]
-            else:
-                return path[len(start) + 1:]
-    else:
-        return path
+    return (
+        bool(start) and
+        (
+            path == start or
+            normcasepath(path).startswith(normcasepath(start + "/"))
+        )
+    )
 
 
 def relativeUniversalPath(path, start):
@@ -900,7 +835,7 @@ def relativeUniversalPath(path, start):
     @return relative path or unchanged path, if path does not start with
         the start path with universal separators (string)
     """
-    return fromNativeSeparators(relativePath(path, start))
+    return fromNativeSeparators(os.path.relpath(path, start))
 
 
 def absolutePath(path, start):
@@ -913,7 +848,7 @@ def absolutePath(path, start):
     @return absolute path (string)
     """
     if not os.path.isabs(path):
-        path = os.path.join(start, path)
+        path = os.path.normpath(os.path.join(start, path))
     return path
 
 
@@ -927,7 +862,7 @@ def absoluteUniversalPath(path, start):
     @return absolute path with native separators (string)
     """
     if not os.path.isabs(path):
-        path = toNativeSeparators(os.path.join(start, path))
+        path = toNativeSeparators(os.path.normpath(os.path.join(start, path)))
     return path
 
 
@@ -947,9 +882,8 @@ def getExecutablePath(file):
             return ""
         
     cur_path = os.path.join(os.curdir, file)
-    if os.path.exists(cur_path):
-        if os.access(cur_path, os.X_OK):
-            return cur_path
+    if os.path.exists(cur_path) and os.access(cur_path, os.X_OK):
+        return cur_path
 
     path = os.getenv('PATH')
     
@@ -984,9 +918,8 @@ def getExecutablePaths(file):
             return []
         
     cur_path = os.path.join(os.curdir, file)
-    if os.path.exists(cur_path):
-        if os.access(cur_path, os.X_OK):
-            paths.append(cur_path)
+    if os.path.exists(cur_path) and os.access(cur_path, os.X_OK):
+        paths.append(cur_path)
 
     path = os.getenv('PATH')
     
@@ -1025,9 +958,8 @@ def getWindowsExecutablePath(file):
     
     for filename in filenames:
         cur_path = os.path.join(os.curdir, filename)
-        if os.path.exists(cur_path):
-            if os.access(cur_path, os.X_OK):
-                return os.path.abspath(cur_path)
+        if os.path.exists(cur_path) and os.access(cur_path, os.X_OK):
+            return os.path.abspath(cur_path)
 
     path = os.getenv('PATH')
     
@@ -1205,10 +1137,7 @@ def direntries(path, filesonly=False, pattern=None, followsymlinks=True,
     @return list of all files and directories in the tree rooted
         at path. The names are expanded to start with path.
     """
-    if filesonly:
-        files = []
-    else:
-        files = [path]
+    files = [] if filesonly else [path]
     try:
         entries = os.listdir(path)
         for entry in entries:
@@ -1255,7 +1184,7 @@ def getDirs(path, excludeDirs):
     """
     try:
         names = os.listdir(path)
-    except EnvironmentError:
+    except OSError:
         return []
 
     dirs = []
@@ -1274,20 +1203,25 @@ def getDirs(path, excludeDirs):
 
     for name in dirs[:]:
         if not os.path.islink(name):
-            dirs = dirs + getDirs(name, excludeDirs)
+            dirs += getDirs(name, excludeDirs)
 
     return dirs
 
 
-def findVolume(volumeName):
+def findVolume(volumeName, findAll=False):
     """
     Function to find the directory belonging to a given volume name.
     
     @param volumeName name of the volume to search for
     @type str
-    @return directory path of the given volume name
-    @rtype str
+    @param findAll flag indicating to get the directories for all volumes
+        starting with the given name (defaults to False)
+    @type bool (optional)
+    @return directory path or list of directory paths for the given volume
+        name
+    @rtype str or list of str
     """
+    volumeDirectories = []
     volumeDirectory = None
     
     if isWindowsPlatform():
@@ -1318,30 +1252,45 @@ def findVolume(volumeName):
         try:
             for disk in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
                 dirpath = "{0}:\\".format(disk)
-                if (os.path.exists(dirpath) and
-                        getVolumeName(dirpath) == volumeName):
-                    volumeDirectory = dirpath
-                    break
+                if os.path.exists(dirpath):
+                    if findAll:
+                        if getVolumeName(dirpath).startswith(volumeName):
+                            volumeDirectories.append(dirpath)
+                    else:
+                        if getVolumeName(dirpath) == volumeName:
+                            volumeDirectory = dirpath
+                            break
         finally:
             ctypes.windll.kernel32.SetErrorMode(oldMode)
     else:
         # we are on a Linux or macOS platform
         for mountCommand in ["mount", "/sbin/mount", "/usr/sbin/mount"]:
-            try:
+            with contextlib.suppress(FileNotFoundError):
                 mountOutput = (
                     subprocess.check_output(mountCommand).splitlines()  # secok
                 )
-                mountedVolumes = [x.split()[2] for x in mountOutput]
-                for volume in mountedVolumes:
-                    if volume.decode("utf-8").endswith(volumeName):
-                        volumeDirectory = volume.decode("utf-8")
+                mountedVolumes = [
+                    x.decode("utf-8").split(" type")[0].split(maxsplit=2)[2]
+                    for x in mountOutput
+                ]
+                if findAll:
+                    for volume in mountedVolumes:
+                        if volumeName in volume:
+                            volumeDirectories.append(volume)
+                    if volumeDirectories:
+                        break
+                else:
+                    for volume in mountedVolumes:
+                        if volume.endswith(volumeName):
+                            volumeDirectory = volume
+                            break
+                    if volumeDirectory:
                         break
-                if volumeDirectory:
-                    break
-            except FileNotFoundError:
-                pass
     
-    return volumeDirectory
+    if findAll:
+        return volumeDirectories
+    else:
+        return volumeDirectory
 
 
 def getTestFileName(fn):
@@ -1362,57 +1311,23 @@ def parseOptionString(s):
     """
     Function used to convert an option string into a list of options.
     
-    @param s option string (string or string)
-    @return list of options (list of strings)
+    @param s option string
+    @type str
+    @return list of options
+    @rtype list of str
     """
-    rx = QRegExp(r"""\s([\w=/-]*"[^"]+"|[\w=/-]*'[^']+'|[^\s]+)""")
     s = re.sub(r"%[A-Z%]", _percentReplacementFunc, s)
-    return parseString(s, rx)
-    
-
-def parseEnvironmentString(s):
-    """
-    Function used to convert an environment string into a list of environment
-    settings.
-    
-    @param s environment string (string)
-    @return list of environment settings (list of strings)
-    """
-    rx = QRegExp(r"""\s(\w+\+?=[^\s]+|\w+="[^"]+"|\w+='[^']+')""")
-    return parseString(s, rx)
-
-
-def parseString(s, rx):
-    """
-    Function used to convert a string into a list.
-    
-    @param s string to be parsed (string)
-    @param rx regex defining the parse pattern (QRegExp)
-    @return list of parsed data (list of strings)
-    """
-    olist = []
-    if not s.startswith(' '):
-        # prepare the  string to fit our pattern
-        s = ' ' + s
-        
-    pos = rx.indexIn(s)
-    while pos != -1:
-        cs = rx.cap(1)
-        if cs.startswith('"') or cs.startswith("'"):
-            cs = cs[1:-1]
-        olist.append(cs)
-        pos += rx.matchedLength()
-        pos = rx.indexIn(s, pos)
-        
-    return olist
+    return shlex.split(s)
 
 
 def _percentReplacementFunc(matchobj):
     """
     Protected function called for replacing % codes.
     
-    @param matchobj matchobject for the code
+    @param matchobj match object for the code
+    @type re.Match
     @return replacement string
+    @rtype str
     """
     return getPercentReplacement(matchobj.group(0))
     
@@ -1421,8 +1336,10 @@ def getPercentReplacement(code):
     """
     Function to get the replacement for code.
     
-    @param code code indicator (string)
-    @return replacement string (string)
+    @param code code indicator
+    @type str
+    @return replacement string
+    @rtype str
     """
     if code in ["C", "%C"]:
         # column of the cursor of the current editor
@@ -1528,9 +1445,8 @@ def getUserName():
     """
     user = getpass.getuser()
     
-    if isWindowsPlatform():
-        if not user:
-            return win32_GetUserName()
+    if isWindowsPlatform() and not user:
+        return win32_GetUserName()
     
     return user
 
@@ -1599,7 +1515,7 @@ def determinePythonVersion(filename, source, editor=None):
     
     @param filename name of the file with extension (str)
     @param source of the file (str)
-    @keyparam editor reference to the editor, if the file is opened
+    @param editor reference to the editor, if the file is opened
         already (Editor object)
     @return Python version if file is Python3 (int)
     """
@@ -1650,11 +1566,11 @@ def determinePythonVersion(filename, source, editor=None):
                 line0 = source.splitlines()[0]
             else:
                 line0 = source[0]
-            if line0.startswith("#!"):
-                if "python3" in line0:
-                    pyVer = 3
-                elif "python" in line0:
-                    pyVer = 3
+            if (
+                line0.startswith("#!") and
+                (("python3" in line0) or ("python" in line0))
+            ):
+                pyVer = 3
         
         if pyVer == 0 and ext in py3Ext:
             pyVer = 3
@@ -1662,6 +1578,25 @@ def determinePythonVersion(filename, source, editor=None):
     return pyVer
 
 
+def rxIndex(rx, txt):
+    """
+    Function to get the index (start position) of a regular expression match
+    within some text.
+    
+    @param rx regular expression object as created by re.compile()
+    @type re.Pattern
+    @param txt text to be scanned
+    @type str
+    @return start position of the match or -1 indicating no match was found
+    @rtype int
+    """
+    match = rx.search(txt)
+    if match is None:
+        return -1
+    else:
+        return match.start()
+
+
 ###############################################################################
 # functions for environment handling
 ###############################################################################
@@ -1677,33 +1612,41 @@ def getEnvironmentEntry(key, default=None):
     @return the requested entry or the default value, if the entry wasn't
         found (string or None)
     """
-    filterRe = QRegExp("^{0}[ \t]*=".format(key))
-    if isWindowsPlatform():
-        filterRe.setCaseSensitivity(Qt.CaseInsensitive)
+    pattern = "^{0}[ \t]*=".format(key)
+    filterRe = (
+        re.compile(pattern, re.IGNORECASE)
+        if isWindowsPlatform() else
+        re.compile(pattern)
+    )
     
     entries = [e for e in QProcess.systemEnvironment()
-               if filterRe.indexIn(e) != -1]
+               if filterRe.search(e) is not None]
     if not entries:
         return default
     
     # if there are multiple entries, just consider the first one
-    ename, val = entries[0].split("=", 1)
-    return val.strip()
+    ename, value = entries[0].split("=", 1)
+    return value.strip()
 
 
 def hasEnvironmentEntry(key):
     """
     Module function to check, if the environment contains an entry.
     
-    @param key key of the requested environment entry (string)
-    @return flag indicating the presence of the requested entry (boolean)
+    @param key key of the requested environment entry
+    @type str
+    @return flag indicating the presence of the requested entry
+    @rtype bool
     """
-    filterRe = QRegExp("^{0}[ \t]*=".format(key))
-    if isWindowsPlatform():
-        filterRe.setCaseSensitivity(Qt.CaseInsensitive)
+    pattern = "^{0}[ \t]*=".format(key)
+    filterRe = (
+        re.compile(pattern, re.IGNORECASE)
+        if isWindowsPlatform() else
+        re.compile(pattern)
+    )
     
     entries = [e for e in QProcess.systemEnvironment()
-               if filterRe.indexIn(e) != -1]
+               if filterRe.search(e) is not None]
     return len(entries) > 0
 
 ###############################################################################
@@ -1815,49 +1758,38 @@ def generatePyQtToolPath(toolname, alternatives=None):
     return exe
 
 ###############################################################################
-# PySide2 utility functions below
+# PySide2/PySide6 utility functions below
 ###############################################################################
 
 
-def generatePySideToolPath(toolname, variant="2"):
+def generatePySideToolPath(toolname, variant=2):
     """
-    Module function to generate the executable path for a PySide2 tool.
+    Module function to generate the executable path for a PySide2/PySide6 tool.
     
     @param toolname base name of the tool
     @type str
-    @param variant indicator for the PySide variant (not used)
-    @type str
-    @return the PySide2 tool path with extension
+    @param variant indicator for the PySide variant
+    @type int or str
+    @return the PySide2/PySide6 tool path with extension
     @rtype str
     """
-    if variant == 1:
-        # no PySide support anymore
-        return ""
-    
     if isWindowsPlatform():
         hasPyside = checkPyside(variant)
         if not hasPyside:
             return ""
         
-        venvName = Preferences.getDebugger("Python3VirtualEnv")
+        venvName = Preferences.getQt("PySide{0}VenvName".format(variant))
+        if not venvName:
+            venvName = Preferences.getDebugger("Python3VirtualEnv")
         interpreter = e5App().getObject(
             "VirtualEnvManager").getVirtualenvInterpreter(venvName)
+        if interpreter == "" or not isinpath(interpreter):
+            interpreter = sys.executable
         prefix = os.path.dirname(interpreter)
-        if toolname in ["pyside2-uic"]:
-            return os.path.join(prefix, "Scripts", toolname + '.exe')
-        else:
-            path = os.path.join(prefix, "Scripts", toolname + '.exe')
-            if os.path.exists(path):
-                return path
-            
-            # report it the old style
-            return os.path.join(
-                prefix, "Lib", "site-packages",
-                "PySide{0}".format(variant),
-                toolname + ".exe")
+        return os.path.join(prefix, "Scripts", toolname + '.exe')
     else:
         # step 1: check, if the user has configured a tools path
-        path = Preferences.getQt("PySide2ToolsDir")
+        path = Preferences.getQt("PySide{0}ToolsDir".format(variant))
         if path:
             return os.path.join(path, toolname)
         
@@ -1866,46 +1798,34 @@ def generatePySideToolPath(toolname, variant="2"):
         if os.path.exists(os.path.join(dirName, toolname)):
             return os.path.join(dirName, toolname)
         
-        # step 3: if it is not 'pyside2-uic' look in the package directory
-        if toolname == "pyside2-uic":
-            return toolname
-        else:
-            import distutils.sysconfig
-            return os.path.join(distutils.sysconfig.get_python_lib(True),
-                                "PySide2", toolname)
+        return toolname
 
 
-def checkPyside(variant="2"):
+def checkPyside(variant=2):
     """
-    Module function to check the presence of PySide2.
+    Module function to check the presence of PySide2/PySide6.
     
-    @param variant indicator for the PySide variant (not used)
-    @type str
-    @return flags indicating the presence of PySide2
+    @param variant indicator for the PySide variant
+    @type int or str
+    @return flags indicating the presence of PySide2/PySide6
     @rtype bool
     """
-    if variant == 1:
-        # no longer supported PySide
-        return [False, False]
-    
-    venvName = Preferences.getDebugger("Python3VirtualEnv")
+    venvName = Preferences.getQt("PySide{0}VenvName".format(variant))
+    if not venvName:
+        venvName = Preferences.getDebugger("Python3VirtualEnv")
     interpreter = e5App().getObject(
         "VirtualEnvManager").getVirtualenvInterpreter(venvName)
     if interpreter == "" or not isinpath(interpreter):
-        return False
-    else:
-        checker = os.path.join(
-            getConfig('ericDir'), "Utilities", "PySideImporter.py")
-        args = [checker, "-" + variant]
-        proc = QProcess()
-        proc.setProcessChannelMode(QProcess.MergedChannels)
-        proc.start(interpreter, args)
-        finished = proc.waitForFinished(30000)
-        if finished:
-            if proc.exitCode() == 0:
-                return True
+        interpreter = sys.executable
     
-    return False
+    checker = os.path.join(
+        getConfig('ericDir'), "Utilities", "PySideImporter.py")
+    args = [checker, "--variant={0}".format(variant)]
+    proc = QProcess()
+    proc.setProcessChannelMode(QProcess.ProcessChannelMode.MergedChannels)
+    proc.start(interpreter, args)
+    finished = proc.waitForFinished(30000)
+    return finished and proc.exitCode() == 0
 
 ###############################################################################
 # Other utility functions below
@@ -1930,10 +1850,7 @@ def generateVersionInfo(linesep='\n'):
     except (ImportError, AttributeError):
         sip_version_str = "sip version not available"
     
-    if sys.maxsize > 2**32:
-        sizeStr = "64-Bit"
-    else:
-        sizeStr = "32-Bit"
+    sizeStr = "64-Bit" if sys.maxsize > 2**32 else "32-Bit"
     
     info = ["Version Numbers:"]
     
@@ -1953,13 +1870,11 @@ def generateVersionInfo(linesep='\n'):
         info.append("  PyQtWebEngine not installed")
     info.append("  QScintilla {0}".format(QSCINTILLA_VERSION_STR))
     info.append("  sip {0}".format(sip_version_str))
-    try:
+    with contextlib.suppress(ImportError):
         from PyQt5 import QtWebEngineWidgets    # __IGNORE_WARNING__
         from WebBrowser.Tools import WebBrowserTools
         chromeVersion = WebBrowserTools.getWebEngineVersions()[0]
         info.append("  WebEngine {0}".format(chromeVersion))
-    except ImportError:
-        pass
     info.append("  {0} {1}".format(Program, Version))
     info.append("")
     info.append("Platform: {0}".format(sys.platform))
@@ -1984,7 +1899,7 @@ def generatePluginsVersionInfo(linesep='\n'):
     info = []
     app = e5App()
     if app is not None:
-        try:
+        with contextlib.suppress(KeyError):
             pm = app.getObject("PluginManager")
             versions = {}
             for pinfo in pm.getPluginInfos():
@@ -1994,8 +1909,6 @@ def generatePluginsVersionInfo(linesep='\n'):
             for pluginModuleName in sorted(versions.keys()):
                 info.append("  {0} {1}".format(
                     pluginModuleName, versions[pluginModuleName]))
-        except KeyError:
-            pass
     
     return linesep.join(info)
 
@@ -2016,10 +1929,9 @@ def generateDistroInfo(linesep='\n'):
             info.append("Distribution Info:")
             for rfile in releaseList:
                 try:
-                    f = open(rfile, "r")
-                    lines = f.read().splitlines()
-                    f.close
-                except IOError:
+                    with open(rfile, "r") as f:
+                        lines = f.read().splitlines()
+                except OSError:
                     continue
                 
                 info.append('  {0}'.format(rfile))
@@ -2063,20 +1975,16 @@ def getSysPath(interpreter):
         getConfig('ericDir'), "Utilities", "GetSysPath.py")
     args = [getSysPath]
     proc = QProcess()
-    proc.setProcessChannelMode(QProcess.MergedChannels)
+    proc.setProcessChannelMode(QProcess.ProcessChannelMode.MergedChannels)
     proc.start(interpreter, args)
     finished = proc.waitForFinished(30000)
-    if finished:
-        if proc.exitCode() == 0:
-            text = proc.readAllStandardOutput()
-            sysPathResult = str(text, "utf-8", "replace").strip()
-            try:
-                sysPath = json.loads(sysPathResult)
-                if "" in sysPath:
-                    sysPath.remove("")
-            except (TypeError, ValueError):
-                # ignore faulty results and return empty list
-                pass
+    if finished and proc.exitCode() == 0:
+        text = proc.readAllStandardOutput()
+        sysPathResult = str(text, "utf-8", "replace").strip()
+        with contextlib.suppress(TypeError, ValueError):
+            sysPath = json.loads(sysPathResult)
+            if "" in sysPath:
+                sysPath.remove("")
     
     return sysPath
 
